% Load the high resolution TCGA data.

function [nprobes, nsamples, probes, samples, data] = load_highRes_data(filename)

% First get the number of probes and samples.
nprobes=0; nsamples=0; clear probes probestrs samples data;
fp=fopen(filename);

% First line contains info about samples.
s=fgets(fp); 
%nsamples=sum(s==9)-1;

% Discard if the last character is a tab.
nsamples=sum(s==9);
tmp=strfind(s,9);
if (tmp(length(tmp))>=(length(s)-3))
nsamples=nsamples-1;
end

for i=1:nsamples
samples{i}=getitemval3(s,i,9);
end

% Get the number of probes.
flag=1; 
while (flag==1)
s=fgets(fp);
if ((length(s)<=0)|(s==-1))
flag=0;
else
nprobes=nprobes+1;
end
end
fclose(fp);


% Get the probe info string and data.

% For mutation data, some entries may havee two numbers separated by commas.
% They denote the states of two chromosomes.

data=NaN*ones(nprobes,nsamples); clear probestrs;
fp=fopen(filename);
s=fgets(fp); 
for i=1:nprobes
probestrs{i}=fscanf(fp,'%s',1);
for j=1:nsamples
data(i,j)=fscanf(fp,'%f',1);
end
end
fclose(fp);

% Parse the probe info.
% If the separator | exists, then parse the probe headers.  Otherwise just report the probes.
clear probes;
for i=1:nprobes
str=probestrs{i};
inds=strfind(str,'|');

if (length(inds)>0)

probes{i}.genename=str(1:(inds(1)-1));
a=inds(1)+1; b=strfind(str,'|'); b=b(2)-1;
if ((str(a)=='X')|(str(a)=='x'))
val=23;
elseif ((str(a)=='Y')|(str(a)=='y'))
val=24;
else
val=atof2(str(a:b));
end
probes{i}.chr=val;

% CNV and mRNA data contain starting and end coordinates of the probe.
% Methylation data just contain one coordinate of the probe.
% Mutation data contain starting and end coordinates of the probe/gene.

% Find the dash in the range of coordinates.
a=b+2; b=strfind(str,'-'); 

if (length(b)>0)
if (length(inds)>=3)
lwb=inds(2)+1; upp=inds(3)-1;
b=b(find((b>=lwb)&(b<=upp)));
else
b=[];
end
end

if (length(b)>0)
b=b(1)-1;
val=atof2(str(a:b));
probes{i}.start=val;
a=b+2; b=inds(3)-1;
val=atof2(str(a:b));
probes{i}.end=val;

else
b=strfind(str,'|'); 
if (length(b)>=3)
b=b(3)-1;
else
b=length(str);
end

% If the probe appears in both X and Y, then the coordinates on X and Y are concatenated by comma.  Choose to report the first coordinate only.
if (length(strfind(str(a:b),','))>0)
ss=strfind(str(a:b),','); b=a+ss(1)-2; 
end

val=atof2(str(a:b));
probes{i}.start=val;
probes{i}.end=val;

end

if (probes{i}.start<=probes{i}.end)
probes{i}.dir=1;
else
probes{i}.dir=-1;
k=probes{i}.start; probes{i}.start=probes{i}.end; probes{i}.end=k;
end

else

probes{i}=str;

end

end

