// Utility functions involved in logistic regression.
// NA values are replaced with -1e6.

# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <time.h>
# include "motif.h"
# include "models.h"
# define max(i,j) (i>j ? i : j)
# define min(i,j) (i<j ? i : j)
# define argmax(i,j) (j>i ? 2 : 1)
#define ITMAX 100 // Maximum allowed number of iterations.
#define EPS 3.0e-7 // Relative accuracy.
#define FPMIN 1.0e-30 // Number near the smallest representable
# define MAXLINEWIDTH 1000000
# define LONGNAMELENGTH 100

extern char * getitemval2(char *s, int ind, int *length);
extern int inlist(int Nv, int *v, int target);
extern void mat_multiply(int ndim1, int ndim2, int ndim3, double *A1, double *A2, double *A3);
extern void mat_inv(int ndim, double *A1, double *A2);
extern double mat_det(int ndim, double *A);
extern double gammln(double xx);
extern double chi2cdf(double val, int deg);
extern double corrcoef(int nsamples, double *x, double *y);
extern char * getitemval3(char *s, int ind, int *length, char sepch);
extern double atof2(char *s, int length);
extern char * getnextitemval3(char *s, int *length, char sepch, int curpos, int *nextpos);

//int nmaxiterations=1000, nperm=10000;
//double NAval=-1e6, epsilon=0.001, infval=1e20;
//double high_thre=0.6, low_thre=0.4;

int nmaxiterations, nperm;
double NAval, epsilon, infval;
double high_thre, low_thre;

double * logistic_regression_parameter_estimation(int nvars, int nsamples, double *data, struct model *model, int *varnstates);
double logistic_regression_ll_evaluation(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas, double *lambdas);
double approx_perm_pval2(int nvars, int nsamples, double *data, struct model *model0, struct model *model1, int *varnstates, double high_thre, double low_thre, int nperm);
void randperm(int n, int *vec);
double logistic_regression_ml_likelihood(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas);
double * ml_llratio_logistic(int nvars, int nsamples, double *data, struct model *model1, struct model *model0, int *varnstates);
double multinomial_log_marginal(int nentries, double *cnts, double *alphas);
double * ml_pairwise_model(int nsamples, double *x, double *y, int mode);
double * ml_pairwise_model2(int nsamples, double *x, double *y, int mode);
void release_memory_model(struct model *model);
int check_consistency(int nvars, struct model *model, int *config, int nstates, int targetvar, int *consistentinvals);
double new_missing_corrcoef(int nsamples, double *x, double *y);
double * additional_power(int nsamples, int noldvars, double *olddata, int *oldnstates, int *oldsigns, double *newvardata, int newnstates, int newsign);
struct matdata * load_highRes_data(char *filename, int *nprobespnt, int *nsamplespnt);
double * load_matrix(char *filename, int *nrowspnt, int *ncolspnt);
struct matdata * load_highRes_data_range(char *filename, int startindex, int endindex, int *nprobespnt, int *nsamplespnt);
struct matdata * load_highRes_data2(char *filename, int nsubprobes, int *probeinds, int *nsamplespnt);
double * ml_pairwise_model_corrected(int nsamples, double *x, double *y, int mode, int nstates);
double * ml_llratio_logistic2(int nvars, int nsamples, double *data, struct model *model1, struct model *model0, int *varnstates);
double logistic_regression_ml_likelihood2(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas);
double * additional_power_corrected(int nsamples, int noldvars, double *olddata, int *oldnstates, int *oldsigns, double *newvardata, int newnstates, int newsign);
double * ml_pairwise_interaction_model_corrected(int nsamples, double *inputvec1, double *inputvec2, double *outputvec, int nstates1, int nstates2, int *direction);
struct integerset * load_cluster(char *filename, int *nclus_int);

// Estimate parameters of the logistic regression model.
double * logistic_regression_parameter_estimation(int nvars, int nsamples, double *data, struct model *model, int *varnstates)
{
  int i, j, k, l, m, n, v, nstates, *varinds, nterms;
  int nit, converge, *state, mm;
  double *dLs, *Hs, val, val2, val3, *tmpmat, *tmpmat2;
  double *lambdas, *oldlambdas, *newlambdas, *jcnts, *realjcnts, fraccnt;

  varinds=malloc(sizeof(int)*nvars);
  for (i=0; i<=(nvars-1); i++)
    *(varinds+i)=i;

  // The last variable of the model is the output variable.
  // If the output variable is not connected to any input variable (null model), then do not estimate parameters.
  if ((model->X+model->nvars-1)->npas<=0)
    {
      lambdas=malloc(sizeof(double)); *lambdas=0;
      return lambdas;
    }

  // Set initial lambdas.
  nterms=(model->X+model->nvars-1)->nterms;
  oldlambdas=malloc(sizeof(double)*nterms);
  for (i=0; i<=(nterms-1); i++)
    *(oldlambdas+i)=1;
  newlambdas=malloc(sizeof(double)*nterms);
  for (i=0; i<=(nterms-1); i++)
    *(newlambdas+i)=1;
  nstates=1;
  for (i=0; i<=(nvars-1); i++)
    nstates*=(*(varnstates+i));

  // Initialize the counts.
  jcnts=malloc(sizeof(double)*nstates*(nvars+1));
  for (i=0; i<=(nstates*(nvars+1)-1); i++)
    *(jcnts+i)=0;
  for (i=0; i<=(nstates-1); i++)
    {
      v=i; j=nvars-1;
      while ((v>0)&&(j>=0))
	{
	  k=*(varinds+j); k=*(varnstates+k);
	  l=(v%k); *(jcnts+i*(nvars+1)+j)=l;
	  v=(v-l)/k; j--;
	}
      *(jcnts+i*(nvars+1)+nvars)=0;
    }

  realjcnts=malloc(sizeof(double)*nstates*(nvars+1));
  for (i=0; i<=(nstates-1); i++)
    for (j=0; j<=nvars; j++)
      *(realjcnts+i*(nvars+1)+j)=*(jcnts+i*(nvars+1)+j);
  for (j=0; j<=(nvars-1); j++)    
    if (*(varnstates+j)==3)
      for (i=0; i<=(nstates-1); i++)
	*(realjcnts+i*(nvars+1)+j)=*(jcnts+i*(nvars+1)+j)-1;    

  // Iteratively apply Newtons' method to estimate parameters.
  nit=0; converge=0;
  dLs=malloc(sizeof(double)*nterms);  
  Hs=malloc(sizeof(double)*nterms*nterms);  
  state=malloc(sizeof(int)*nvars);  
  while (converge==0)
    {
      double *fs, *yfs, Z;

      // Calculate the gradient and Hessian.
      for (i=0; i<=(nterms-1); i++)
	*(dLs+i)=0;
      for (i=0; i<=(nterms-1); i++)
	for (j=0; j<=(nterms-1); j++)
	  *(Hs+i*nterms+j)=0;
     
      fs=malloc(sizeof(double)*nterms);
      yfs=malloc(sizeof(double)*nterms);

      for (n=0; n<=(nsamples-1); n++)
	{
	  // Count the fraction occurrences of each state.
	  for (i=0; i<=(nstates-1); i++)
	    *(jcnts+i*(nvars+1)+nvars)=0;
	  i=n;
	  for (j=0; j<=(nstates-1); j++)
	    {
	      val=1;
	      for (k=0; k<=(nvars-1); k++)
		*(state+k)=*(jcnts+j*(nvars+1)+k);
	      for (k=0; k<=(nvars-1); k++)
		{
		  int varind=*(varinds+k);
		  double entry=*(data+varind*nsamples+i);
		  int subnstates=*(varnstates+varind);

		  // cdf value is NA (<-1e6): equal mass on each state.
		  if (entry<=NAval)
		    val/=(double)(subnstates);
		  // cdf value >= 1: state=(nstates-1).
		  else if ((entry>=1)&&(*(state+k)!=(subnstates-1)))
		    val=0;
		  // cdf value <= 0: state=0.
		  else if ((entry<=0)&&(*(state+k)!=0))
		    val=0;
		  // cdf value is in (0,1): apply probabilistic quantization transformation.
		  else if ((entry>0)&&(entry<1))
		    {
		      double val1, val2;
		      val1=entry/(1-log(entry));
		      val2=(1-entry)/(1-log(1-entry));
		      if ((subnstates==3)&&(*(state+k)==0))
			val*=val2;
		      else if ((subnstates==3)&&(*(state+k)==2))
			val*=val1;
		      else if ((subnstates==3)&&(*(state+k)==1))
			val*=(1-val1-val2);
		      else if ((subnstates==2)&&(*(state+k)==0))
			val*=(1-val1);
		      else if ((subnstates==2)&&(*(state+k)==1))
			val*=val1;
		    }
		}
	      
	      *(jcnts+j*(nvars+1)+nvars)+=val;	      
	    }
	  for (j=0; j<=(nstates-1); j++)
	    *(realjcnts+j*(nvars+1)+nvars)=*(jcnts+j*(nvars+1)+nvars);
	

	  // Evaluate the value of each feature.	  
	  for (i=0; i<=(nterms-1); i++)
	    *(fs+i)=0;	  
	  for (i=0; i<=(nterms-1); i++)
	    *(yfs+i)=0;
	  
	  for (mm=0; mm<=(nstates-1); mm++)
	    {
	      int y, z;
	      
	      for (i=0; i<=(nvars-1); i++)
		*(state+i)=*(realjcnts+mm*(nvars+1)+i);
	      fraccnt=*(realjcnts+mm*(nvars+1)+nvars);
	      y=*(state+nvars-1);
	      	     
	      for (i=0; i<=(nterms-1); i++)
		{
		  int *subvarinds, nsubvarinds, *substate1, *substate2, nrows;

		  // Count the truth table of variables in the ith term.
		  subvarinds=((model->X+model->nvars-1)->terms+i)->varinds;
		  nsubvarinds=((model->X+model->nvars-1)->terms+i)->nvars;
		  nrows=((model->X+model->nvars-1)->terms+i)->nrows;
		  
		  // Get the substate on the term.
		  substate1=malloc(sizeof(int)*nsubvarinds);
		  for (j=0; j<=(nsubvarinds-1); j++)
		    *(substate1+j)=*(state+*(subvarinds+j));
		  substate2=malloc(sizeof(int)*nsubvarinds);
		  		 
		  // Look up the truth table value.
		  j=0; k=-1;
		  while ((j<=(nrows-1))&&(k<0))
		    {
		      for (l=0; l<=(nsubvarinds-1); l++)
			*(substate2+l)=*(((model->X+model->nvars-1)->terms+i)->table+j*(nsubvarinds+1)+l);
		      m=0;
		      for (l=0; l<=(nsubvarinds-1); l++)
			if (*(substate1+l)!=*(substate2+l))
			  m++;
		      if (m==0)
			k=j;		      

		      j++;
		    }
		  
		  z=*(((model->X+model->nvars-1)->terms+i)->table+k*(nsubvarinds+1)+nsubvarinds); 
		  *(fs+i)+=(fraccnt*z);
		  *(yfs+i)+=(fraccnt*z*y);		 

		  free(substate1); free(substate2);
		 
		}
	    }
	       

	  // Evaluate \sum_{i}\lambda_{i}f_{i}.
	  Z=0;
	  for (i=0; i<=(nterms-1); i++)
	    {
	      val=*(oldlambdas+i)*(*(fs+i));
	      Z+=val;
	    }
	  
	  // Add the terms of the gradient.
	  for (i=0; i<=(nterms-1); i++)
	    {
	      *(dLs+i)+=*(yfs+i);
	      *(dLs+i)-=(*(fs+i)*(exp(Z)-exp(-1*Z))/(1+exp(Z)+exp(-1*Z)));
	    }
		 
	  // Add the terms of the Hessian.
	  val2=exp(Z)+exp(-1*Z);
	  val3=exp(Z)-exp(-1*Z);
	  val=(val2*val2+val2-val3*val3)/((1+val2)*(1+val2));
	  tmpmat=malloc(sizeof(double)*nterms*nterms);
	  mat_multiply(nterms,1,nterms,fs,fs,tmpmat);

	  for (i=0; i<=(nterms*nterms-1); i++)
	    *(tmpmat+i)*=val;
	  for (i=0; i<=(nterms*nterms-1); i++)
	    *(Hs+i)-=*(tmpmat+i);
    	  
	  free(tmpmat);
	}

      // Stop when |det(Hs)|<epsilon.
      val=mat_det(nterms,Hs); val=fabs(val);


      if (val<epsilon)
	{
	  converge=1;	  
	  for (i=0; i<=(nterms-1); i++)
	    *(newlambdas+i)=*(oldlambdas+i);
	}

      // Otherwise apply Newton's method to estimate lambdas.
      else
	{
	  tmpmat=malloc(sizeof(double)*nterms*nterms);
	  tmpmat2=malloc(sizeof(double)*nterms);	  
	  mat_inv(nterms,Hs,tmpmat);
	  mat_multiply(nterms,nterms,1,tmpmat,dLs,tmpmat2);	  
	  for (i=0; i<=(nterms-1); i++)
	    *(newlambdas+i)=*(oldlambdas+i)-*(tmpmat2+i);
	  free(tmpmat); free(tmpmat2);
	  
	  // If the parameter values are too large, then stick to the old parameter values.
	  k=0;
	  for (i=0; i<=(nterms-1); i++)
	    {
	      val=*(newlambdas+i); val=fabs(val);
	      if (val>100)
		k++;
	    }
	  if (k>0)
	    {
	      converge=1;	  
	      for (i=0; i<=(nterms-1); i++)
		*(newlambdas+i)=*(oldlambdas+i);
	    }
	}

      nit++;


      // Check whether lambdas converge.
      if (converge==0)
	{
	  val=0;
	  for (i=0; i<=(nterms-1); i++)
	    val+=(*(newlambdas+i)-*(oldlambdas+i))*(*(newlambdas+i)-*(oldlambdas+i));
	  val=sqrt(val);
	  if ((val<=epsilon)||(nit>=nmaxiterations))	    
	    converge=1;
	  else	    
	    for (i=0; i<=(nterms-1); i++)
	      *(oldlambdas+i)=*(newlambdas+i);
	}


      free(fs); free(yfs);
    }

  // Report lambdas.
  lambdas=malloc(sizeof(double)*nterms);
  for (i=0; i<=(nterms-1); i++)
    *(lambdas+i)=*(newlambdas+i);


  // If any lambda value < 0, then set the value to 0.
  for (i=0; i<=(nterms-1); i++)
    if (*(lambdas+i)<0)
      *(lambdas+i)=0;


  free(dLs); free(Hs); free(state); free(varinds);
  free(oldlambdas); free(newlambdas);
  free(jcnts); free(realjcnts);
  

  return lambdas;

}

    
// Evaluate the log likelihood score of a logistic regression model given the parameters.
double logistic_regression_ll_evaluation(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas, double *lambdas)
{
  int i, j, k, l, m, n, v, nstates, *state, *varinds, nterms, mm;
  double val, ll=0;

  varinds=malloc(sizeof(int)*nvars);
  for (i=0; i<=(nvars-1); i++)
    *(varinds+i)=i;
  state=malloc(sizeof(int)*nvars);

  // If the output variable is not connected to any input variable (null model), then calculate the multinomial likelihood.
  if ((model->X+model->nvars-1)->npas<=0)
    {
      double *cnts;

      nstates=*(varnstates+nvars-1);
      cnts=malloc(sizeof(double)*nstates);
      for (i=0; i<=(nstates-1); i++)
	*(cnts+i)=0;
      for (i=0; i<=(nsamples-1); i++)
	{	  
	  val=*(data+(nvars-1)*nsamples+i);	  	  
	  if (val<=NAval)
	    {
	      for (j=0; j<=(nstates-1); j++)
		*(cnts+j)+=(double)1/(double)nstates;
	    }
	  else if (val>=1)
	    *(cnts+nstates-1)+=1;
	  else if (val<=0)
	    *(cnts+0)+=1;
	  else
	    {
	      double val1, val2;
	      val1=val/(1-log(val)); 
	      val2=(1-val)/(1-log(1-val));
	      if (nstates==3)
		{
		  *(cnts+0)+=val2; *(cnts+2)+=val1; *(cnts+1)+=(1-val1-val2);
		}
	      else
		{
		  *(cnts+0)+=(1-val1); *(cnts+1)+=val1;
		}
	    }
	}      
      ll=multinomial_log_marginal(nstates,cnts,alphas);      

      free(cnts);
    }

  // If the output variable is connected to some input variables, then calculate the logistic regression model likelihood.
  else
    {
      double *jcnts, *realjcnts;
      double *fs, *yfs, val1, val2;
            
      // Set the real counts table.
      nstates=1;
      for (i=0; i<=(nvars-1); i++)
	nstates*=(*(varnstates+i));

      jcnts=malloc(sizeof(double)*nstates*(nvars+1));
      for (i=0; i<=(nstates*(nvars+1)-1); i++)
	*(jcnts+i)=0;
      for (i=0; i<=(nstates-1); i++)
	{
	  v=i; j=nvars-1;
	  while ((v>0)&&(j>=0))
	    {
	      k=*(varinds+j); k=*(varnstates+k);
	      l=(v%k); *(jcnts+i*(nvars+1)+j)=l;
	      v=(v-l)/k; j--;
	    }
	  *(jcnts+i*(nvars+1)+nvars)=0;
	}
     
      realjcnts=malloc(sizeof(double)*nstates*(nvars+1));
      for (i=0; i<=(nstates-1); i++)
	for (j=0; j<=nvars; j++)
	  *(realjcnts+i*(nvars+1)+j)=*(jcnts+i*(nvars+1)+j);
      for (j=0; j<=(nvars-1); j++)    
	if (*(varnstates+j)==3)
	  for (i=0; i<=(nstates-1); i++)
	    *(realjcnts+i*(nvars+1)+j)=*(jcnts+i*(nvars+1)+j)-1;    
	      
      // Calculate the fractional counts.
      ll=0; nterms=(model->X+model->nvars-1)->nterms;

      fs=malloc(sizeof(double)*nterms);
      yfs=malloc(sizeof(double)*nterms);
  

      for (n=0; n<=(nsamples-1); n++)
	{
	  // Count the fraction occurrences of each state.
	  for (i=0; i<=(nstates-1); i++)
	    *(jcnts+i*(nvars+1)+nvars)=0;
	  i=n;
	  for (j=0; j<=(nstates-1); j++)
	    {
	      val=1;
	      for (k=0; k<=(nvars-1); k++)
		*(state+k)=*(jcnts+j*(nvars+1)+k);
	      for (k=0; k<=(nvars-1); k++)
		{
		  int varind=*(varinds+k);
		  double entry=*(data+varind*nsamples+i);
		  int subnstates=*(varnstates+varind);

		  // cdf value is NA (<-1e6): equal mass on each state.
		  if (entry<=NAval)
		    val/=(double)(subnstates);
		  // cdf value >= 1: state=(nstates-1).
		  else if ((entry>=1)&&(*(state+k)!=(subnstates-1)))
		    val=0;
		  // cdf value <= 0: state=0.
		  else if ((entry<=0)&&(*(state+k)!=0))
		    val=0;
		  // cdf value is in (0,1): apply probabilistic quantization transformation.
		  else if ((entry>0)&&(entry<1))
		    {
		      double val1, val2;
		      val1=entry/(1-log(entry));
		      val2=(1-entry)/(1-log(1-entry));
		      if ((subnstates==3)&&(*(state+k)==0))
			val*=val2;
		      else if ((subnstates==3)&&(*(state+k)==2))
			val*=val1;
		      else if ((subnstates==3)&&(*(state+k)==1))
			val*=(1-val1-val2);
		      else if ((subnstates==2)&&(*(state+k)==0))
			val*=(1-val1);
		      else if ((subnstates==2)&&(*(state+k)==1))
			val*=val1;
		    }
		}
	      *(jcnts+j*(nvars+1)+nvars)+=val;	      
	    }
	  for (j=0; j<=(nstates-1); j++)
	    *(realjcnts+j*(nvars+1)+nvars)=*(jcnts+j*(nvars+1)+nvars);
	
	  
	  // Evaluate the value of each feature.	  
	  for (i=0; i<=(nterms-1); i++)
	    *(fs+i)=0;	  
	  for (i=0; i<=(nterms-1); i++)
	    *(yfs+i)=0;
	  
	  for (mm=0; mm<=(nstates-1); mm++)
	    {
	      int y, z;
	      double fraccnt;

	      for (i=0; i<=(nvars-1); i++)
		*(state+i)=*(realjcnts+mm*(nvars+1)+i);
	      fraccnt=*(realjcnts+mm*(nvars+1)+nvars);
	      y=*(state+nvars-1);
	      
	      for (i=0; i<=(nterms-1); i++)
		{
		  int *subvarinds, nsubvarinds, *substate1, *substate2, nrows;
		  subvarinds=((model->X+model->nvars-1)->terms+i)->varinds;
		  nsubvarinds=((model->X+model->nvars-1)->terms+i)->nvars;
		  nrows=((model->X+model->nvars-1)->terms+i)->nrows;
		  
		  // Get the substate on the term.
		  substate1=malloc(sizeof(int)*nsubvarinds);
		  for (j=0; j<=(nsubvarinds-1); j++)
		    *(substate1+j)=*(state+*(subvarinds+j));
		  substate2=malloc(sizeof(int)*nsubvarinds);
		  
		  // Look up the truth table value.
		  j=0; k=-1;
		  while ((j<=(nrows-1))&&(k<0))
		    {
		      for (l=0; l<=(nsubvarinds-1); l++)
			*(substate2+l)=*(((model->X+model->nvars-1)->terms+i)->table+j*(nsubvarinds+1)+l);
		      m=0;
		      for (l=0; l<=(nsubvarinds-1); l++)
			if (*(substate1+l)!=*(substate2+l))
			  m++;
		      if (m==0)
			k=j;
		      j++;
		    }
		  
		  z=*(((model->X+model->nvars-1)->terms+i)->table+k*(nsubvarinds+1)+nsubvarinds);
		  *(fs+i)+=(fraccnt*z);
		  *(yfs+i)+=(fraccnt*z*y);		  

		  free(substate1); free(substate2);
		}
	    }
	  
	  // Evaluate \sum_{i}\lambda_{i}yfs_{i}-log(1+exp(\sum_{i}\lambda_{i}fs_{i})+exp(-\sum_{i}\lambda_{i}fs_{i})+1).
	  val1=0;
	  for (i=0; i<=(nterms-1); i++)
	    val1+=(*(lambdas+i)*(*(yfs+i)));
	  val2=0;
	  for (i=0; i<=(nterms-1); i++)
	    val2+=(*(lambdas+i)*(*(fs+i)));
	  ll+=(val1-log(1+exp(val2)+exp(-1*val2)));
	}
      
      free(jcnts); free(realjcnts); 
      free(fs); free(yfs);
    }

  free(varinds); free(state);

  return ll;
}


// Approximated permutation p-value of multiple variables.
// Fix all but one variables according to the null and active models.
// Permute the target variable and calculate the probability of getting <= T inconsistent data points.

double approx_perm_pval2(int nvars, int nsamples, double *data, struct model *model0, struct model *model1, int *varnstates, double high_thre, double low_thre, int nperm)
{
  int i, j, k, l, m, n, v, nstates, nterms;
  int targetvar, *config, *consistenttargetvars, *template, consistent;
  double val, pval=1, nC, nC2, *permdata;
  
  
  // Find the target variable to permute.
  targetvar=-1; i=0;
  while ((i<(model1->X+model1->nvars-1)->npas)&&(targetvar<0))
    {
      j=*((model1->X+model1->nvars-1)->pas+i);
      if (inlist((model0->X+model0->nvars-1)->npas,(model0->X+model0->nvars-1)->pas,j)==0)
	targetvar=j;
      i++;
    }
  nstates=*(varnstates+targetvar);
  
  // Count the number of contradictory data points.
  nC=0; consistenttargetvars=malloc(sizeof(int)*nstates*nsamples);
  config=malloc(sizeof(int)*nvars);
  for (i=0; i<=(nstates*nsamples-1); i++)
    *(consistenttargetvars+i)=0;
  template=malloc(sizeof(int)*nstates);

  
  for (i=0; i<=(nsamples-1); i++)
    {
      // Quantize the configuration.
      for (j=0; j<=(nvars-1); j++)
	*(config+j)=0;
      for (j=0; j<=(nvars-1); j++)
	{
	  val=*(data+j*nsamples+i);
	  if (*(varnstates+j)==3)
	    {
	      if (val<=NAval)
		*(config+j)=0;
	      else if (val>=high_thre)
		*(config+j)=1;
	      else if (val<=low_thre)
		*(config+j)=-1;
	    }
	  else
	    {	      
	      if (val>=high_thre)
		*(config+j)=1;
	    }
	}

      // Check if the configuration is consistent with model1.
      consistent=check_consistency(nvars,model1,config,nstates,targetvar,template);

      for (j=0; j<=(nstates-1); j++)
	*(consistenttargetvars+j*nsamples+i)=*(template+j);
      
      if (consistent==0)
	nC++;

    }

  // Recompute the number of inconsistent data points.
  nC=0;
  for (i=0; i<=(nsamples-1); i++)
    {
      val=*(data+targetvar*nsamples+i);
      k=0;
      if (nstates==3)
	{
	  if (val<=NAval)
	    k=1;
	  else if (val>=high_thre)
	    k=2;
	  else if (val<=low_thre)
	    k=0;
	  else
	    k=1;
	}
      else
	{	  
	  if (val>=high_thre)
	    k=1;
	  else
	    k=0;
	}
      if (*(consistenttargetvars+k*nsamples+i)==0)
	nC++;

    }


  // Perform random permutations on the target variable and evaluate the p-value.
  permdata=malloc(sizeof(double)*nvars*nsamples);
  for (i=0; i<=(nvars*nsamples-1); i++)
    *(permdata+i)=*(data+i);
    
  
  pval=0; 
  for (n=0; n<=(nperm-1); n++)
    {
      int *vec;

      // Permute the data randomly.
      vec=malloc(sizeof(int)*nsamples);
      for (i=0; i<=(nsamples-1); i++)
	*(vec+i)=i;
      randperm(nsamples,vec);     
      
      for (i=0; i<=(nsamples-1); i++)
	{
	  j=*(vec+i);
	  *(permdata+targetvar*nsamples+i)=*(data+targetvar*nsamples+j);
	}

      // Count the number of inconsistent data points.
      // Just need to check if the quantized target variable value falls in the consistent input values.
      nC2=0;
      for (i=0; i<=(nsamples-1); i++)
	{
	  val=*(permdata+targetvar*nsamples+i);
	  k=0;
	  if (nstates==3)
	    {
	      if (val>=high_thre)
		k=2;
	      else if (val<=low_thre)
		k=0;
	      else
		k=1;
	    }
	  else
	    {
	      if (val>=high_thre)
		k=1;
	      else
		k=0;
	    }
	  if (*(consistenttargetvars+k*nsamples+i)==0)
	    nC2++;
	}

      if (nC2<=nC)
	pval++;
      free(vec);
    }

  pval/=(double)nperm;
    
  free(config); free(consistenttargetvars); free(template); free(permdata);
    

  return pval;
}


// Randomly permute an array of consecutive numbers from 0 to n-1.
void randperm(int n, int *vec)
{
  int i, j, k, *subset, *covered;  
  for (i=0; i<=(n-1); i++)
    *(vec+i)=-1;
  subset=malloc(sizeof(int)*n);
  covered=malloc(sizeof(int)*n);
  for (i=0; i<=(n-1); i++)
    {     
      for (j=0; j<=(n-1); j++)
	*(covered+j)=0;
      for (j=0; j<=(n-1); j++)
	if (*(vec+j)>=0)
	  *(covered+*(vec+j))=1;
      
      k=0;
      for (j=0; j<=(n-1); j++)
	{
	  if (*(covered+j)==0)
	   {
	      k++; *(subset+k-1)=j;
	    }
	} 


      j=rand(); j=j%k; j=*(subset+j);
      *(vec+i)=j;
    }
  free(subset); free(covered);
}


// Evaluate the log likelihood score of logistic regression model using the maximum likelihood parameters.
double logistic_regression_ml_likelihood(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas)
{
  int i, j, k;
  double *lambdas, logL=0;

  // Estimate parameters.
  lambdas=logistic_regression_parameter_estimation(nvars,nsamples,data,model,varnstates); 

  // If any component of lambdas<0, then return 0.
  k=0;
  for (i=0; i<=((model->X+model->nvars-1)->nterms-1); i++)
    if (*(lambdas+i)<0)
      k++;
  if (k>0)
    logL=-1*infval;
  else
    logL=logistic_regression_ll_evaluation(nvars,nsamples,data,model,varnstates,alphas,lambdas);

  free(lambdas);
 

  return logL;
}



// Calculate the log likelihood ratio and p-value of two logistic regression models.
// The first returned value is log likelihood, the second returned value is pvalue.
double * ml_llratio_logistic(int nvars, int nsamples, double *data, struct model *model1, struct model *model0, int *varnstates)
{
  int i, j, k, l, m, n, nstates, naddinputs, *addinputs, ff, d;
  double *cdf1, *cnts, val, val1, val2, *nulldata;
  double logratio=0, pval=1, *retvec, *alphas, logL0, logL1, tmp0=1e10, tmp1=1e10, shiftlogratio;
  double adlogratio, pval1=1, pval2=1;  

  // If any additional input or output variable concentrates on one state, then discard it.
  cdf1=data+(nvars-1)*nsamples;
  nstates=*(varnstates+nvars-1);
  cnts=malloc(sizeof(double)*nstates);
  for (i=0; i<=(nstates-1); i++)
    *(cnts+i)=0;
  for (i=0; i<=(nsamples-1); i++)
    {
      val=*(cdf1+i);
      if (val<=NAval)
	*(cnts+0)+=0;
      else if (val>=1)
	*(cnts+nstates-1)+=1;
      else if (val<=0)
	*(cnts+0)+=1;
      else
	{
	  val1=val/(1-log(val)); val2=(1-val)/(1-log(1-val));
	  if (nstates==3)
	    {
	      *(cnts+0)+=val2;
	      *(cnts+2)+=val1;
	      *(cnts+1)+=(1-val1-val2);
	    }
	  else
	    {
	      *(cnts+0)+=(1-val1);
	      *(cnts+1)+=val1;
	    }
	}
    }

  retvec=malloc(sizeof(double)*2);
  

  if ((nstates==3)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+2)<=(0.05*nsamples))))
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }
  else if ((nstates==2)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+1)<=(0.05*nsamples))))
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }

  addinputs=malloc(sizeof(int)); ff=0; naddinputs=0;
  for (i=0; i<=((model1->X+model1->nvars-1)->npas-1); i++)
    {
      j=*((model1->X+model1->nvars-1)->pas+i);
      if (inlist((model0->X+model0->nvars-1)->npas,(model0->X+model0->nvars-1)->pas,j)==0)
	{
	  naddinputs++; *(addinputs+naddinputs-1)=j;
	}
    }
  
  for (n=0; n<=(naddinputs-1); n++)
    {
      cdf1=data+*(addinputs+n)*nsamples;
      nstates=*(varnstates+*(addinputs+n));
      for (i=0; i<=(nstates-1); i++)
	*(cnts+i)=0;
      for (i=0; i<=(nsamples-1); i++)
	{
	  val=*(cdf1+i);
	  if (val<=NAval)
	    *(cnts+0)+=0;
	  else if (val>=1)
	    *(cnts+nstates-1)+=1;
	  else if (val<=0)
	    *(cnts+0)+=1;
	  else
	    {
	      val1=val/(1-log(val)); val2=(1-val)/(1-log(1-val));
	      if (nstates==3)
		{
		  *(cnts+0)+=val2;
		  *(cnts+2)+=val1;
		  *(cnts+1)+=(1-val1-val2);
		}
	      else
		{
		  *(cnts+0)+=(1-val1);
		  *(cnts+2)+=val1;
		}
	    }
	}

      if ((nstates==3)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+2)<=(0.05*nsamples))))
	ff=1;
      else if ((nstates==2)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+1)<=(0.05*nsamples))))
	ff=1;
    }
      
  if (ff==1)
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }


  // Evaluate the logratio and pvalue of the models.
  alphas=malloc(sizeof(double)*3);
  for (i=0; i<=2; i++)
    *(alphas+i)=1;
    
  logL0=logistic_regression_ml_likelihood(nvars,nsamples,data,model0,varnstates,alphas);
  logL1=logistic_regression_ml_likelihood(nvars,nsamples,data,model1,varnstates,alphas);
  logratio=logL1-logL0; 
    

  if ((logL0<=(-1*infval))&&(logL1<=(-1*infval)))
    logratio=0;  

  // Evaluate the log likelihood ratio of the unobserved data.
  nulldata=malloc(sizeof(double)*nvars*nsamples);
  for (i=0; i<=(nvars*nsamples-1); i++)
    *(nulldata+i)=NAval;
    
  tmp0=logistic_regression_ml_likelihood(nvars,nsamples,nulldata,model0,varnstates,alphas);  
  tmp1=logistic_regression_ml_likelihood(nvars,nsamples,nulldata,model1,varnstates,alphas);
    

  shiftlogratio=tmp1-tmp0; 
  tmp1=max(tmp1,tmp0);
  if (tmp1<=(-1*infval))
    tmp1=logL0;
  
  // If model1 has only one input, then choose logL0=tmp1.  The multinomial distribution of output variables is not the right null model.
  if ((model1->X+model1->nvars-1)->npas==1)
    logratio=logL1-min(logL0,tmp1);

  // Calculate the chi2 p-value.
  d=(model1->X+model1->nvars-1)->npas-(model0->X+model0->nvars-1)->npas;
  adlogratio=logratio;


  if (adlogratio>0)
    pval1=1-chi2cdf(2*adlogratio,d);    
  else
    pval1=1;
  

  // Calculate the permutation p-value.
  if (pval1<1)
    pval2=approx_perm_pval2(nvars,nsamples,data,model0,model1,varnstates,high_thre,low_thre,nperm);
  else
    pval2=1;
  

  pval=max(pval1,pval2);

  if (logratio<=(-1*infval))
    logratio=0;

  *(retvec+0)=logratio; *(retvec+1)=pval;

  free(cnts); free(addinputs); free(alphas); free(nulldata);

  return retvec;
}


// Evaluate the log marginal likelihood of a multinomial distribution.
// If the counts and pseudocounts are integers, then calculate log factorial.  Otherwise calculate gamma function.
double multinomial_log_marginal(int nentries, double *cnts, double *alphas)
{
  int i, j, k, l, m, n, nn, n1, n2;
  double total, val, val2;

  val=0; nn=nentries;
  total=0;
  for (i=0; i<=(nentries-1); i++)    
    total+=(*(cnts+i)+*(alphas+i));
  
  for (i=0; i<=(nentries-1); i++)
    {
      val2=*(cnts+i)+*(alphas+i)+1;
      val2=gammln(val2);	  
      val+=val2;
    }
  val2=total+nentries;
  val2=gammln(val2);      
  val-=val2;

  return val;
}


// Check whether a configuration is consistent with the model prediction.
// Check whether the configuration matches each truth table in the model.
// Also report the consistent input values for each data point.
int check_consistency(int nvars, struct model *model, int *config, int nstates, int targetvar, int *consistentinvals)
{
  int i, j, k, l, m, n, p, *valset, *vec, ll, consistent;
  int *newconfig, *consistentinvals2, flag, flag2, inflag, inflag2, *subconfig;

  valset=malloc(sizeof(int)*nstates);
  if (nstates==3)
    {
      for (i=0; i<=(nstates-1); i++)
	*(valset+i)=i-1;
    }
  else
    {
      for (i=0; i<=(nstates-1); i++)
	*(valset+i)=i;
    }

  newconfig=malloc(sizeof(int)*nvars);
  for (i=0; i<=(nvars-1); i++)
    *(newconfig+i)=*(config+i);
  subconfig=malloc(sizeof(int)*(nvars+1));
  vec=malloc(sizeof(int)*(nvars+1));

  consistentinvals2=malloc(sizeof(int)*nstates);
  for (i=0; i<=(nstates-1); i++)
    {
      *(consistentinvals+i)=0;
      *(consistentinvals2+i)=0;
    }
  consistent=0;
  for (j=0; j<=(nstates-1); j++)
    {
      k=*(valset+j); *(newconfig+targetvar)=k;     
      flag=1; l=0; inflag=1;
      while ((l<=((model->X+model->nvars-1)->nterms-1))&&(flag==1))
	{
	  int nsubrows, nsubvars;

	  nsubrows=((model->X+model->nvars-1)->terms+l)->nrows;
	  nsubvars=((model->X+model->nvars-1)->terms+l)->nvars;	  
	  for (n=0; n<=(nsubvars-1); n++)	    
	    *(subconfig+n)=*(newconfig+*(((model->X+model->nvars-1)->terms+l)->varinds+n));
	  *(subconfig+nsubvars)=*(newconfig+nvars-1);
	  flag2=0; m=0; inflag2=0;
	  while ((m<=(nsubrows-1))&&(flag2==0))
	    {
	      for (p=0; p<=nsubvars; p++)
		{
		  *(vec+p)=*(((model->X+model->nvars-1)->terms+l)->table+m*(nsubvars+1)+p);
		  if (*(vec+p)>0)
		    *(vec+p)=1;
		  else if (*(vec+p)<0)
		    *(vec+p)=-1;
		  else
		    *(vec+p)=0;
		}
	      
	      //flag2=0;
	      if (*(vec+nsubvars)!=0)
		{
		  flag2=1; p=0;
		  while ((p<=nsubvars)&&(flag2==1))
		    {
		      if (*(subconfig+p)!=*(vec+p))
			flag2=0;
		      p++;
		    }
		  if (flag2==1)
		    inflag2=1;
		}
	      if ((flag2==0)&&(*(vec+nsubvars)==0))
		{
		  flag2=1; p=0;
		  while ((p<=(nsubvars-1))&&(flag2==1))
		    {
		      if (*(subconfig+p)!=*(vec+p))
			flag2=0;
		      p++;
		    }
		}

	      m++;
	    }
	  if (flag2==0)
	    flag=0;
	  if (inflag2==0)
	    inflag=0;
	  l++;
	}

      if ((flag==1)&&(*(config+targetvar)==k))
	consistent=1;
      if (inflag==1)
	*(consistentinvals+j)=1;
      if (flag==1)
	*(consistentinvals2+j)=1;     
    }


  k=0;
  for (i=0; i<=(nstates-1); i++)
    k+=*(consistentinvals+i);
  if (k==0)
    {
      for (i=0; i<=(nstates-1); i++)
	*(consistentinvals+i)=*(consistentinvals2+i);
    }

  free(newconfig); free(valset); free(vec);
  free(consistentinvals2); free(subconfig);

  return consistent;
}


// Release memory of model data structure.
void release_memory_model(struct model *model)
{
  int i, j, k;

  for (i=0; i<=(model->nvars-1); i++)
    {
      if ((model->X+i)->npas>0)
	free((model->X+i)->pas);
      if ((model->X+i)->nterms>0)
	{
	  for (j=0; j<=((model->X+i)->nterms-1); j++)
	    {
	      free(((model->X+i)->terms+j)->varinds);
	      free(((model->X+i)->terms+j)->table);
	    }
	  free((model->X+i)->terms);
	}
    }
  free(model);
}


// Evaluate the log likelihood ratio and pvalue of a pairwise model.  Combination of model1_1 and model1_2_3.  Also incur the permutation p-value test.
// mode=1: x is an activator of y.  mode=2: x is a repressor of y.
double * ml_pairwise_model(int nsamples, double *x, double *y, int mode)
{
  int i, j, k, l, m, n, *varnstates;
  double *data, *retvec;
  struct model *model0, *model1;

  // Construct the models.
  data=malloc(sizeof(double)*nsamples*2);
  for (i=0; i<=(nsamples-1); i++)
    {
      *(data+i)=*(x+i);
      *(data+i+nsamples)=*(y+i);
    }
  varnstates=malloc(sizeof(int)*2);
  for (i=0; i<=1; i++)
    *(varnstates+i)=3;  
  
  model1=malloc(sizeof(struct model));
  model1->nvars=2;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  (model1->X+0)->varind=0;
  (model1->X+0)->npas=0;
  (model1->X+0)->nterms=0;
  (model1->X+1)->varind=1;
  (model1->X+1)->npas=1;
  (model1->X+1)->pas=malloc(sizeof(int)*(model1->X+1)->npas);
  *((model1->X+1)->pas+0)=0;
  (model1->X+1)->nterms=1;
  (model1->X+1)->terms=malloc(sizeof(struct term)*(model1->X+1)->nterms);
  ((model1->X+1)->terms+0)->nvars=1;
  ((model1->X+1)->terms+0)->varinds=malloc(sizeof(int)*((model1->X+1)->terms+0)->nvars);
  *(((model1->X+1)->terms+0)->varinds+0)=0;
  ((model1->X+1)->terms+0)->nrows=3;
  ((model1->X+1)->terms+0)->table=malloc(sizeof(int)*(((model1->X+1)->terms+0)->nvars+1)*((model1->X+1)->terms+0)->nrows);
  n=0;
  for (i=-1; i<=1; i++)
    {
      *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+0)=i;
      if (mode==1)
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=i;
      else
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=-1*i;
      n++;
    }

  model0=malloc(sizeof(struct model));
  model0->nvars=2;
  model0->X=malloc(sizeof(struct varinfo)*model0->nvars);
  (model0->X+0)->varind=0;
  (model0->X+0)->npas=0;
  (model0->X+0)->nterms=0;
  (model0->X+1)->varind=1;
  (model0->X+1)->npas=0;
  (model0->X+1)->nterms=0;

  // Calculate the log likelihood and pvalue of the model.
  retvec=ml_llratio_logistic(2,nsamples,data,model1,model0,varnstates);
  
  // Release memory.
  release_memory_model(model1); 
  release_memory_model(model0); 
  free(data); free(varnstates);

  return retvec;
}


// Evaluate the log likelihood ratio and pvalue of a pairwise model.  Combination of model1_1 and model1_2_3.  Also incur the permutation p-value test.
// mode=1: x is an activator of y.  mode=2: x is a repressor of y.
// Difference from pairwise_model.m: input is mutation with binary state.
// mode=1: x in an activator of y, 0 input has 0 output.
// mode=2: x in an repressor of y, 0 input has 0 output.
// mode=3: x in an activator of y, 0 input has -1 output.
// mode=4: x in an repressor of y, 0 input has 1 output.
double * ml_pairwise_model2(int nsamples, double *x, double *y, int mode)
{
  int i, j, k, l, m, n, *varnstates;
  double *data, *retvec;
  struct model *model0, *model1;

  // Construct the models.
  data=malloc(sizeof(double)*nsamples*2);
  for (i=0; i<=(nsamples-1); i++)
    {
      *(data+i)=*(x+i);
      *(data+i+nsamples)=*(y+i);
    }
  varnstates=malloc(sizeof(int)*2);
  *(varnstates+0)=2; *(varnstates+1)=3;

  model1=malloc(sizeof(struct model));
  model1->nvars=2;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  (model1->X+0)->varind=0;
  (model1->X+0)->npas=0;
  (model1->X+0)->nterms=0;
  (model1->X+1)->varind=1;
  (model1->X+1)->npas=1;
  (model1->X+1)->pas=malloc(sizeof(int)*(model1->X+1)->npas);
  *((model1->X+1)->pas+0)=0;
  (model1->X+1)->nterms=1;
  (model1->X+1)->terms=malloc(sizeof(struct term)*(model1->X+1)->nterms);
  ((model1->X+1)->terms+0)->nvars=1;
  ((model1->X+1)->terms+0)->varinds=malloc(sizeof(int)*((model1->X+1)->terms+0)->nvars);
  *(((model1->X+1)->terms+0)->varinds+0)=0;
  ((model1->X+1)->terms+0)->nrows=2;
  ((model1->X+1)->terms+0)->table=malloc(sizeof(int)*(((model1->X+1)->terms+0)->nvars+1)*((model1->X+1)->terms+0)->nrows);
  n=0;
  for (i=0; i<=1; i++)
    {
      *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+0)=i;
      if (mode==1)
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=i;
      else if (mode==2)
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=-1*i;
      else if (mode==3)
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=2*i-1;
      else if (mode==4)
	*(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=1-2*i;
      n++;
    }
  
  model0=malloc(sizeof(struct model));
  model0->nvars=2;
  model0->X=malloc(sizeof(struct varinfo)*model0->nvars);
  (model0->X+0)->varind=0;
  (model0->X+0)->npas=0;
  (model0->X+0)->nterms=0;
  (model0->X+1)->varind=1;
  (model0->X+1)->npas=0;
  (model0->X+1)->nterms=0;
    

  // Calculate the log likelihood and pvalue of the model.
  retvec=ml_llratio_logistic(2,nsamples,data,model1,model0,varnstates);

  // Release memory.
  release_memory_model(model1); 
  release_memory_model(model0); 
  free(data); free(varnstates);

  return retvec;
}


// Calculate pairwise correlation coefficient with NA entries.
// Fill the NA entries with the mean of the vector.
double new_missing_corrcoef(int nsamples, double *x, double *y)
{
  int i, j, k;
  double val, *x2, *y2;
  
  // Calculate the mean of each vector.
  val=0; k=0;
  for (i=0; i<=(nsamples-1); i++)    
    {
      if (*(x+i)>NAval)
	{
	  val+=*(x+i); k++;	  
	}
    }
  
  if (k>0)
    val/=(double)k;

  x2=malloc(sizeof(double)*nsamples);
  for (i=0; i<=(nsamples-1); i++)
    {
      if (*(x+i)>NAval)
	*(x2+i)=*(x+i);
      else
	*(x2+i)=val;
    }
  
  val=0; k=0;
  for (i=0; i<=(nsamples-1); i++)    
    {
      if (*(y+i)>NAval)
	{
	  val+=*(y+i); k++;
	}
    }
    
  if (k>0)
    val/=(double)k;
  
  y2=malloc(sizeof(double)*nsamples);
  for (i=0; i<=(nsamples-1); i++)
    {
      if (*(y+i)>NAval)
	*(y2+i)=*(y+i);
      else
	*(y2+i)=val;
    }
  
  val=corrcoef(nsamples,x2,y2);
  free(x2); free(y2);

  return val;
}


/* Given input variables of an existing, calculate the log likelihood ratio and pvalue of adding another variable to fit the data.
   Inputs: data, input variables of the existing model, input signs of the existing model, the sign of the additional variable.
   Output: log ratio and pvalue of the joint model compared to the existing model.
*/

double * additional_power(int nsamples, int noldvars, double *olddata, int *oldnstates, int *oldsigns, double *newvardata, int newnstates, int newsign)
{
  int i, j, k, l, m, n, *varnstates;
  double *data, *retvec;
  struct model *model1, *model2;
  
  data=malloc(sizeof(double)*(noldvars+1)*nsamples);  
  for (n=0; n<=(noldvars-2); n++)    
    for (i=0; i<=(nsamples-1); i++)
      *(data+n*nsamples+i)=*(olddata+n*nsamples+i);  
  for (i=0; i<=(nsamples-1); i++)
    *(data+(noldvars-1)*nsamples+i)=*(newvardata+i);
  for (i=0; i<=(nsamples-1); i++)
    *(data+noldvars*nsamples+i)=*(olddata+(noldvars-1)*nsamples+i);
 
  varnstates=malloc(sizeof(int)*(noldvars+1));
  for (n=0; n<=(noldvars-2); n++)
    *(varnstates+n)=*(oldnstates+n);
  *(varnstates+noldvars-1)=newnstates;
  *(varnstates+noldvars)=*(oldnstates+noldvars-1);
 
  // Build the existing model.
  // The last variable of oldvars is the output variable.
  model1=malloc(sizeof(struct model));
  model1->nvars=noldvars+1;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  for (i=0; i<=(model1->nvars-1); i++)
    {
      (model1->X+i)->varind=i;
      (model1->X+i)->npas=0;
      (model1->X+i)->nterms=0;
    }
  (model1->X+model1->nvars-1)->npas=noldvars-1;
  (model1->X+model1->nvars-1)->pas=malloc(sizeof(int)*(model1->X+model1->nvars-1)->npas);
  for (i=0; i<=((model1->X+model1->nvars-1)->npas-1); i++)
    *((model1->X+model1->nvars-1)->pas+i)=i;
  (model1->X+model1->nvars-1)->nterms=noldvars-1;
  (model1->X+model1->nvars-1)->terms=malloc(sizeof(struct term)*(model1->X+model1->nvars-1)->nterms);
  for (n=0; n<=((model1->X+model1->nvars-1)->nterms-1); n++)
    {
      int n1, n2, funcsign, starti;
      ((model1->X+model1->nvars-1)->terms+n)->nvars=1;
      ((model1->X+model1->nvars-1)->terms+n)->varinds=malloc(sizeof(int));
      *(((model1->X+model1->nvars-1)->terms+n)->varinds+0)=n;
      ((model1->X+model1->nvars-1)->terms+n)->nrows=*(varnstates+n);
      n1=*(varnstates+n); n2=2; 
      ((model1->X+model1->nvars-1)->terms+n)->table=malloc(sizeof(int)*n1*n2);
      if (*(varnstates+n)==3)
	starti=-1;
      else
	starti=0;
      k=0; funcsign=*(oldsigns+n);
      for (i=starti; i<=1; i++)
	{
	  k++; 	  
	  *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+0)=i;
	  if (*(varnstates+n)==3)
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=i*funcsign;
	  else
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=funcsign*(2*i-1);
	}
    }
  
  // Build the joint model.
  // Include the additional variable.
  model2=malloc(sizeof(struct model));
  model2->nvars=model1->nvars;
  model2->X=malloc(sizeof(struct varinfo)*model2->nvars);
  for (n=0; n<=(model2->nvars-1); n++)
    {
      (model2->X+n)->varind=n;
      (model2->X+n)->npas=(model1->X+n)->npas;
      (model2->X+n)->nterms=(model1->X+n)->nterms;
      if ((model2->X+n)->npas>0)
	{
	  int n1, n2, funcsign, starti;

	  (model2->X+n)->npas+=1;
	  (model2->X+n)->pas=malloc(sizeof(int)*(model2->X+n)->npas);
	  for (i=0; i<=((model1->X+n)->npas-1); i++)
	    *((model2->X+n)->pas+i)=*((model1->X+n)->pas+i);
	  *((model2->X+n)->pas+(model2->X+n)->npas-1)=noldvars-1;
	  (model2->X+n)->nterms=(model1->X+n)->nterms+1;
	  (model2->X+n)->terms=malloc(sizeof(struct term)*(model2->X+n)->nterms);
	  for (i=0; i<=((model1->X+n)->nterms-1); i++)
	    {
	      ((model2->X+n)->terms+i)->nvars=((model1->X+n)->terms+i)->nvars;
	      ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int)*((model2->X+n)->terms+i)->nvars);
	      for (j=0; j<=(((model2->X+n)->terms+i)->nvars-1); j++)
		*(((model2->X+n)->terms+i)->varinds+j)=*(((model1->X+n)->terms+i)->varinds+j);
	      ((model2->X+n)->terms+i)->nrows=((model1->X+n)->terms+i)->nrows;
	      n1=((model2->X+n)->terms+i)->nrows; n2=2; 
	      
	      ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2);
	      for (j=0; j<=(n1*n2-1); j++)
		*(((model2->X+n)->terms+i)->table+j)=*(((model1->X+n)->terms+i)->table+j);		
	    }
	  
	  // Add the new variable in another term.
	  i=(model2->X+n)->nterms-1;
	  ((model2->X+n)->terms+i)->nvars=1;
	  ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int));
	  *(((model2->X+n)->terms+i)->varinds+0)=noldvars-1;
	  ((model2->X+n)->terms+i)->nrows=*(varnstates+noldvars-1);
	  n1=*(varnstates+noldvars-1); n2=2;
	  ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2);
	  if (n1==3)
	    starti=-1;
	  else
	    starti=0;
	  k=0; funcsign=newsign;
	  for (j=starti; j<=1; j++)
	    {
	      k++; 	  
	      *(((model2->X+n)->terms+i)->table+(k-1)*n2+0)=j;
	      if (n1==3)
		*(((model2->X+n)->terms+i)->table+(k-1)*n2+1)=j*funcsign;
	      else
		*(((model2->X+n)->terms+i)->table+(k-1)*n2+1)=funcsign*(2*j-1);
	    }	  
	}
    }
    
  // Calculate the log likelihood and pvalue of the model.
  retvec=ml_llratio_logistic(noldvars+1,nsamples,data,model2,model1,varnstates);

  // Release memory.
  release_memory_model(model1); 
  release_memory_model(model2); 
  free(data); free(varnstates);

  return retvec;
}

// Load a TCGA dataset.
// Report the information about probes and samples as well.
struct matdata * load_highRes_data(char *filename, int *nprobespnt, int *nsamplespnt)
{
  int i, j, k, l, m, n, nprobes, nsamples, length, curpos=0, nextpos=-1, issnp=-1;
  char *s, *item, ch, *samples;
  double val, *data;
  struct genemirseginfo *probes;
  struct matdata *retdata;
  FILE *fp;

  // Open the field.
  if ((fp=fopen(filename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",filename);
      return -1;
    }

  // Get the number of probes and samples.  
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  fgets(s,MAXLINEWIDTH,fp);
  nsamples=0; samples=malloc(sizeof(char)); i=0; 
  while ((i<strlen(s))&&((ch=*(s+i))!=0x0a))
    {
      if (ch==0x09)
	nsamples++;
      i++;
    } 
  samples=malloc(sizeof(char)*nsamples*LONGNAMELENGTH);
  for (n=0; n<=(nsamples-1); n++)
    {
      item=getitemval3(s,n+1,&length,0x09);
      strncpy(samples+n*LONGNAMELENGTH,item,length);
      *(samples+n*LONGNAMELENGTH+length)='\0';
      free(item);
    }
  if (length<=0)
    nsamples-=1;
 
  nprobes=0; probes=malloc(sizeof(struct genemirseginfo));
  data=malloc(sizeof(double));
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      // Check whether the data is a SNP file.
      // The probe header of a SNP contains the following info: SNP name, gene name, chromosome number, coordinate.
      // The probe header of a non-SNP contains the following info: gene name, chromosome number, start coordinate, end coordinate.
      if (issnp<0)
	{
	  item=getitemval3(s,1,&length,'|');	 
	  i=0;
	  while ((i<=(length-1))&&(issnp<0))
	    {
	      if (((ch=*(item+i))<'0')||(ch>'9'))
		issnp=1;
	      i++;
	    }

	  if (issnp<0)
	    issnp=0;

	  free(item);
	}
      
      // Non-SNP data.
      if (issnp==0)
	{      
	  nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);        
	  (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	  *((probes+nprobes-1)->snpname+0)='\0';
	  item=getitemval3(s,0,&length,'|');	  
	  (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	  strncpy((probes+nprobes-1)->name,item,length);
	  *((probes+nprobes-1)->name+length)='\0';
	  if (length>0)
	    free(item);      
	  item=getitemval3(s,1,&length,'|');      
	  val=atof2(item,length);
	  (probes+nprobes-1)->chr=(int)val;
	  if (length>0)
	    free(item);
	  item=getitemval3(s,2,&length,'|');
	  val=atof2(item,length);
	  (probes+nprobes-1)->startcoord=(int)val;
	  if (length>0)
	    free(item);
	  item=getitemval3(s,3,&length,'|');
	  val=atof2(item,length);
	  (probes+nprobes-1)->endcoord=(int)val;
	  if (length>0)
	    free(item);
	}

      // SNP data.
      else
	{
	  nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);
	  item=getitemval3(s,0,&length,'|');	  
	  (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	  strncpy((probes+nprobes-1)->snpname,item,length);
	  *((probes+nprobes-1)->snpname+length)='\0';
	  if (length>0)
	    free(item);      	  	  
	  item=getitemval3(s,1,&length,'|');	  
	  (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	  strncpy((probes+nprobes-1)->name,item,length);
	  *((probes+nprobes-1)->name+length)='\0';
	  if (length>0)
	    free(item);      
	  item=getitemval3(s,2,&length,'|');      
	  val=atof2(item,length);
	  (probes+nprobes-1)->chr=(int)val;
	  if (length>0)
	    free(item);
	  item=getitemval3(s,3,&length,'|');
	  val=atof2(item,length);
	  (probes+nprobes-1)->startcoord=(int)val;
	  (probes+nprobes-1)->endcoord=(int)val;
	  if (length>0)
	    free(item);	  	  
	}


      // Data entries.
      
      item=getitemval3(s,0,&length,0x09);
      curpos=length+1; nextpos=-1; free(item);

      data=realloc(data,sizeof(double)*nprobes*nsamples);      
      for (i=0; i<=(nsamples-1); i++)
	{
	  item=getnextitemval3(s,&length,0x09,curpos,&nextpos);
	  curpos=nextpos;

	  if (strncmp(item,"NaN",3)==0)	    
	    *(data+(nprobes-1)*nsamples+i)=NAval;
	  else
	    {
	      val=atof2(item,length);
	      *(data+(nprobes-1)*nsamples+i)=val;
	    }
	  free(item);
	}

    }
  fclose(fp);
 
  retdata=malloc(sizeof(struct matdata));
  retdata->nprobes=nprobes; retdata->nsamples=nsamples;
  retdata->probes=probes; retdata->samples=samples; retdata->data=data;
  *nprobespnt=nprobes; *nsamplespnt=nsamples;

  return retdata;
}


// Load a matrix file of real numbers.
double * load_matrix(char *filename, int *nrowspnt, int *ncolspnt)
{
  int i, j, k, l, m, n, nrows, ncols, length, curpos, nextpos;
  char *s, *item, ch;
  double *mat, val;
  FILE *fp;
  
  // Open the file.
  if ((fp=fopen(filename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",filename);
      return -1;
    }
  
  // Get the number of rows and columns.
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  fgets(s,MAXLINEWIDTH,fp); ncols=0; i=0;
  while ((i<strlen(s))&&((ch=*(s+i))!=0x0a))
    {
      if (ch==0x09)
	ncols++;
      i++;
    } 
  ncols++;
  
  nrows=1; mat=malloc(sizeof(double)*ncols); curpos=0;
  for (n=0; n<=(ncols-1); n++)
    {
      item=getnextitemval3(s,&length,0x09,curpos,&nextpos);
      curpos=nextpos;

      val=atof2(item,length); free(item);
      *(mat+n)=val; 
    }  

  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      nrows++; mat=realloc(mat,sizeof(double)*nrows*ncols);
      curpos=0;
      for (n=0; n<=(ncols-1); n++)
	{
	  item=getnextitemval3(s,&length,0x09,curpos,&nextpos);
	  curpos=nextpos;

	  val=atof2(item,length); free(item);
	  *(mat+(nrows-1)*ncols+n)=val; 
	}
    }
  fclose(fp);
  *nrowspnt=nrows; *ncolspnt=ncols;

  return mat;
}


// Load a TCGA dataset.
// Report the information about probes and samples as well.
// Only report the rows within the range of [startindex,endindex].
struct matdata * load_highRes_data_range(char *filename, int startindex, int endindex, int *nprobespnt, int *nsamplespnt)
{
  int i, j, k, l, m, n, nprobes, nsamples, length, curpos=0, nextpos=-1, issnp=-1, cnt;
  char *s, *item, ch, *samples;
  double val, *data;
  struct genemirseginfo *probes;
  struct matdata *retdata;
  FILE *fp;

  // Open the field.
  if ((fp=fopen(filename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",filename);
      return -1;
    }

  // Get the number of probes and samples.  
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  fgets(s,MAXLINEWIDTH,fp);
  nsamples=0; samples=malloc(sizeof(char)); i=0; 
  while ((i<strlen(s))&&((ch=*(s+i))!=0x0a))
    {
      if (ch==0x09)
	nsamples++;
      i++;
    } 
  samples=malloc(sizeof(char)*nsamples*LONGNAMELENGTH);
  for (n=0; n<=(nsamples-1); n++)
    {
      item=getitemval3(s,n+1,&length,0x09);
      strncpy(samples+n*LONGNAMELENGTH,item,length);
      *(samples+n*LONGNAMELENGTH+length)='\0';
      free(item);
    }
  if (length<=0)
    nsamples-=1;


  nprobes=0; probes=malloc(sizeof(struct genemirseginfo));
  data=malloc(sizeof(double)); cnt=0;
  while (((fgets(s,MAXLINEWIDTH,fp))!=NULL)&&(cnt<=endindex))
    {
      if ((cnt>=startindex)&&(cnt<=endindex))
	{

	  // Check whether the data is a SNP file.
	  // The probe header of a SNP contains the following info: SNP name, gene name, chromosome number, coordinate.
	  // The probe header of a non-SNP contains the following info: gene name, chromosome number, start coordinate, end coordinate.
	  if (issnp<0)
	    {
	      item=getitemval3(s,1,&length,'|');	      	      
	      i=0;
	      while ((i<=(length-1))&&(issnp<0))
		{
		  if (((ch=*(item+i))<'0')||(ch>'9'))
		    issnp=1;
		  i++;
		}

	      if (issnp<0)
		issnp=0;

	      free(item);
	    }
	  
	  // Non-SNP data.
	  if (issnp==0)
	    {      
	      nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);        
	      (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	      *((probes+nprobes-1)->snpname+0)='\0';
	      item=getitemval3(s,0,&length,'|');	  
	      (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->name,item,length);
	      *((probes+nprobes-1)->name+length)='\0';
	      if (length>0)
		free(item);      
	      item=getitemval3(s,1,&length,'|');      
	      val=atof2(item,length);
	      (probes+nprobes-1)->chr=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,2,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->startcoord=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,3,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->endcoord=(int)val;
	      if (length>0)
		free(item);
	    }
	  
	  // SNP data.
	  else
	    {
	      nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);
	      item=getitemval3(s,0,&length,'|');	  
	      (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->snpname,item,length);
	      *((probes+nprobes-1)->snpname+length)='\0';
	      if (length>0)
		free(item);      	  	  
	      item=getitemval3(s,1,&length,'|');	  
	      (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->name,item,length);
	      *((probes+nprobes-1)->name+length)='\0';
	      if (length>0)
		free(item);      
	      item=getitemval3(s,2,&length,'|');      
	      val=atof2(item,length);
	      (probes+nprobes-1)->chr=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,3,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->startcoord=(int)val;
	      (probes+nprobes-1)->endcoord=(int)val;
	      if (length>0)
		free(item);	  
	      	     
	    }


	  // Data entries.
	  
	  item=getitemval3(s,0,&length,0x09);
	  curpos=length+1; nextpos=-1; free(item);
	  
	  data=realloc(data,sizeof(double)*nprobes*nsamples);      
	  for (i=0; i<=(nsamples-1); i++)
	    {
	      item=getnextitemval3(s,&length,0x09,curpos,&nextpos);
	      curpos=nextpos;
	      
	      if (strncmp(item,"NaN",3)==0)	    
		*(data+(nprobes-1)*nsamples+i)=NAval;
	      else
		{
		  val=atof2(item,length);
		  *(data+(nprobes-1)*nsamples+i)=val;
		}
	      free(item);
	    }
	}

      cnt++;
    }
  fclose(fp);
 
  retdata=malloc(sizeof(struct matdata));
  retdata->nprobes=nprobes; retdata->nsamples=nsamples;
  retdata->probes=probes; retdata->samples=samples; retdata->data=data;
  *nprobespnt=nprobes; *nsamplespnt=nsamples;

  return retdata;
}


// Load a TCGA dataset.
// Report the information about probes and samples as well.
// Difference from load_highRes_data: only load the rows specified by the input.
struct matdata * load_highRes_data2(char *filename, int nsubprobes, int *probeinds, int *nsamplespnt)
{
  int i, j, k, l, m, n, nprobes, nsamples, length, curpos=0, nextpos=-1, issnp=-1, cnt;
  char *s, *item, ch, *samples;
  double val, *data;
  struct genemirseginfo *probes;
  struct matdata *retdata;
  FILE *fp;

  // Open the field.
  if ((fp=fopen(filename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",filename);
      return -1;
    }

  // Get the number of probes and samples.  
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  fgets(s,MAXLINEWIDTH,fp);
  nsamples=0; samples=malloc(sizeof(char)); i=0; 
  while ((i<strlen(s))&&((ch=*(s+i))!=0x0a))
    {
      if (ch==0x09)
	nsamples++;
      i++;
    } 
  samples=malloc(sizeof(char)*nsamples*LONGNAMELENGTH);
  for (n=0; n<=(nsamples-1); n++)
    {
      item=getitemval3(s,n+1,&length,0x09);
      strncpy(samples+n*LONGNAMELENGTH,item,length);
      *(samples+n*LONGNAMELENGTH+length)='\0';
      free(item);
    }
  if (length<=0)
    nsamples-=1;

  nprobes=0; probes=malloc(sizeof(struct genemirseginfo));
  data=malloc(sizeof(double)); cnt=0;
  while (((fgets(s,MAXLINEWIDTH,fp))!=NULL)&&(nprobes<nsubprobes))
    {
      // Extract the rows specified by probeinds.
      if (inlist(nsubprobes,probeinds,cnt)==1)
	{

	  // Check whether the data is a SNP file.
	  // The probe header of a SNP contains the following info: SNP name, gene name, chromosome number, coordinate.
	  // The probe header of a non-SNP contains the following info: gene name, chromosome number, start coordinate, end coordinate.
	  if (issnp<0)
	    {
	      item=getitemval3(s,1,&length,'|');	      	      
	      i=0;
	      while ((i<=(length-1))&&(issnp<0))
		{
		  if (((ch=*(item+i))<'0')||(ch>'9'))
		    issnp=1;
		  i++;
		}

	      if (issnp<0)
		issnp=0;

	      free(item);
	    }
	  
	  // Non-SNP data.
	  if (issnp==0)
	    {      
	      nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);        
	      (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	      *((probes+nprobes-1)->snpname+0)='\0';
	      item=getitemval3(s,0,&length,'|');	  
	      (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->name,item,length);
	      *((probes+nprobes-1)->name+length)='\0';
	      if (length>0)
		free(item);      
	      item=getitemval3(s,1,&length,'|');      
	      val=atof2(item,length);
	      (probes+nprobes-1)->chr=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,2,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->startcoord=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,3,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->endcoord=(int)val;
	      if (length>0)
		free(item);
	    }
	  
	  // SNP data.
	  else
	    {
	      nprobes++; probes=realloc(probes,sizeof(struct genemirseginfo)*nprobes);
	      item=getitemval3(s,0,&length,'|');	  
	      (probes+nprobes-1)->snpname=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->snpname,item,length);
	      *((probes+nprobes-1)->snpname+length)='\0';
	      if (length>0)
		free(item);      	  	  
	      item=getitemval3(s,1,&length,'|');	  
	      (probes+nprobes-1)->name=malloc(sizeof(char)*LONGNAMELENGTH);
	      strncpy((probes+nprobes-1)->name,item,length);
	      *((probes+nprobes-1)->name+length)='\0';
	      if (length>0)
		free(item);      
	      item=getitemval3(s,2,&length,'|');      
	      val=atof2(item,length);
	      (probes+nprobes-1)->chr=(int)val;
	      if (length>0)
		free(item);
	      item=getitemval3(s,3,&length,'|');
	      val=atof2(item,length);
	      (probes+nprobes-1)->startcoord=(int)val;
	      (probes+nprobes-1)->endcoord=(int)val;
	      if (length>0)
		free(item);	  	  
	    }
	  
	  
	  // Data entries.
	  	 
	  item=getitemval3(s,0,&length,0x09);
	  curpos=length+1; nextpos=-1; free(item);
	  
	  data=realloc(data,sizeof(double)*nprobes*nsamples);      
	  for (i=0; i<=(nsamples-1); i++)
	    {
	      item=getnextitemval3(s,&length,0x09,curpos,&nextpos);
	      curpos=nextpos;
	      
	      if (strncmp(item,"NaN",3)==0)	    
		*(data+(nprobes-1)*nsamples+i)=NAval;
	      else
		{
		  val=atof2(item,length);
		  *(data+(nprobes-1)*nsamples+i)=val;
		}
	      free(item);
	    }
	}
      cnt++;
    }
  fclose(fp);
 
  retdata=malloc(sizeof(struct matdata));
  retdata->nprobes=nprobes; retdata->nsamples=nsamples;
  retdata->probes=probes; retdata->samples=samples; retdata->data=data;
  *nsamplespnt=nsamples;

  return retdata;
}


// Evaluate the log likelihood ratio and pvalue of a pairwise model.  Combination of model1_1 and model1_2_3.  Also incur the permutation p-value test.
// Combine ml_pairwise_model and ml_pairwise_model2.
// Also, correct the errors when the null model is an independent model.
// output nstates=3:
// mode=1: x is an activator of y.  mode=2: x is a repressor of y.
// output nstates=2:
// mode=1: x in an activator of y, 0 input has 0 output.
// mode=2: x in an repressor of y, 0 input has 0 output.
// mode=3: x in an activator of y, 0 input has -1 output.
// mode=4: x in an repressor of y, 0 input has 1 output.
double * ml_pairwise_model_corrected(int nsamples, double *x, double *y, int mode, int nstates)
{
  int i, j, k, l, m, n, *varnstates;
  double *data, *retvec;
  struct model *model0, *model1;

  // Construct the models.
  data=malloc(sizeof(double)*nsamples*2);
  for (i=0; i<=(nsamples-1); i++)
    {
      *(data+i)=*(x+i);
      *(data+i+nsamples)=*(y+i);
    }
  varnstates=malloc(sizeof(int)*2);
  *(varnstates+0)=nstates;
  *(varnstates+1)=3;
    
  model1=malloc(sizeof(struct model));
  model1->nvars=2;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  (model1->X+0)->varind=0;
  (model1->X+0)->npas=0;
  (model1->X+0)->nterms=0;
  (model1->X+1)->varind=1;
  (model1->X+1)->npas=1;
  (model1->X+1)->pas=malloc(sizeof(int)*(model1->X+1)->npas);
  *((model1->X+1)->pas+0)=0;
  (model1->X+1)->nterms=1;
  (model1->X+1)->terms=malloc(sizeof(struct term)*(model1->X+1)->nterms);
  ((model1->X+1)->terms+0)->nvars=1;
  ((model1->X+1)->terms+0)->varinds=malloc(sizeof(int)*((model1->X+1)->terms+0)->nvars);
  *(((model1->X+1)->terms+0)->varinds+0)=0;
  ((model1->X+1)->terms+0)->nrows=nstates;
  ((model1->X+1)->terms+0)->table=malloc(sizeof(int)*(((model1->X+1)->terms+0)->nvars+1)*((model1->X+1)->terms+0)->nrows);
  if (nstates==3)
    {
      n=0;
      for (i=-1; i<=1; i++)
	{
	  *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+0)=i;
	  if (mode==1)
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=i;
	  else
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=-1*i;
	  n++;
	}
    }
  else
    {
      n=0;
      for (i=0; i<=1; i++)
	{
	  *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+0)=i;
	  if (mode==1)
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=i;
	  else if (mode==2)
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=-1*i;
	  else if (mode==3)
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=2*i-1;
	  else if (mode==4)
	    *(((model1->X+1)->terms+0)->table+n*(((model1->X+1)->terms+0)->nvars+1)+1)=1-2*i;
	  n++;
	}
    }

  model0=malloc(sizeof(struct model));
  model0->nvars=2;
  model0->X=malloc(sizeof(struct varinfo)*model0->nvars);
  (model0->X+0)->varind=0;
  (model0->X+0)->npas=0;
  (model0->X+0)->nterms=0;
  (model0->X+1)->varind=1;
  (model0->X+1)->npas=0;
  (model0->X+1)->nterms=0;

  // Calculate the log likelihood and pvalue of the model.
  retvec=ml_llratio_logistic2(2,nsamples,data,model1,model0,varnstates);
  
  // Release memory.
  release_memory_model(model1); 
  release_memory_model(model0); 
  free(data); free(varnstates);

  return retvec;
}


// Calculate the log likelihood ratio and p-value of two logistic regression models.
// The first returned value is log likelihood, the second returned value is pvalue.
// Difference from ml_llratio_logistic: correct the evaluation when the null model is independent or when some lambda values < 0.
double * ml_llratio_logistic2(int nvars, int nsamples, double *data, struct model *model1, struct model *model0, int *varnstates)
{
  int i, j, k, l, m, n, nstates, naddinputs, *addinputs, ff, d, interaction;
  double *cdf1, *cnts, val, val1, val2, *nulldata;
  double logratio=0, pval=1, *retvec, *alphas, logL0, logL1, tmp0=1e10, tmp1=1e10, shiftlogratio;
  double adlogratio, pval1=1, pval2=1;  

  // If any additional input or output variable concentrates on one state, then discard it.
  cdf1=data+(nvars-1)*nsamples;
  nstates=*(varnstates+nvars-1);
  cnts=malloc(sizeof(double)*nstates);
  for (i=0; i<=(nstates-1); i++)
    *(cnts+i)=0;
  for (i=0; i<=(nsamples-1); i++)
    {
      val=*(cdf1+i);
      if (val<=NAval)
	*(cnts+0)+=0;
      else if (val>=1)
	*(cnts+nstates-1)+=1;
      else if (val<=0)
	*(cnts+0)+=1;
      else
	{
	  val1=val/(1-log(val)); val2=(1-val)/(1-log(1-val));
	  if (nstates==3)
	    {
	      *(cnts+0)+=val2;
	      *(cnts+2)+=val1;
	      *(cnts+1)+=(1-val1-val2);
	    }
	  else
	    {
	      *(cnts+0)+=(1-val1);
	      *(cnts+1)+=val1;
	    }
	}
    }

  retvec=malloc(sizeof(double)*2);
  

  if ((nstates==3)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+2)<=(0.05*nsamples))))
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }
  else if ((nstates==2)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+1)<=(0.05*nsamples))))
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }

  addinputs=malloc(sizeof(int)); ff=0; naddinputs=0;
  for (i=0; i<=((model1->X+model1->nvars-1)->npas-1); i++)
    {
      j=*((model1->X+model1->nvars-1)->pas+i);
      if (inlist((model0->X+model0->nvars-1)->npas,(model0->X+model0->nvars-1)->pas,j)==0)
	{
	  naddinputs++; *(addinputs+naddinputs-1)=j;
	}
    }
  
  for (n=0; n<=(naddinputs-1); n++)
    {
      cdf1=data+*(addinputs+n)*nsamples;
      nstates=*(varnstates+*(addinputs+n));
      for (i=0; i<=(nstates-1); i++)
	*(cnts+i)=0;
      for (i=0; i<=(nsamples-1); i++)
	{
	  val=*(cdf1+i);
	  if (val<=NAval)
	    *(cnts+0)+=0;
	  else if (val>=1)
	    *(cnts+nstates-1)+=1;
	  else if (val<=0)
	    *(cnts+0)+=1;
	  else
	    {
	      val1=val/(1-log(val)); val2=(1-val)/(1-log(1-val));
	      if (nstates==3)
		{
		  *(cnts+0)+=val2;
		  *(cnts+2)+=val1;
		  *(cnts+1)+=(1-val1-val2);
		}
	      else
		{
		  *(cnts+0)+=(1-val1);
		  *(cnts+2)+=val1;
		}
	    }
	}

      if ((nstates==3)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+2)<=(0.05*nsamples))))
	ff=1;
      else if ((nstates==2)&&((*(cnts+0)<=(0.05*nsamples))||(*(cnts+1)<=(0.05*nsamples))))
	ff=1;
    }
      
  if (ff==1)
    {
      logratio=0; pval=1;
      *(retvec+0)=logratio; *(retvec+1)=pval;
      free(cnts);
      return retvec;
    }


  // Evaluate the logratio and pvalue of the models.
  alphas=malloc(sizeof(double)*3);
  for (i=0; i<=2; i++)
    *(alphas+i)=1;
  
    
  logL0=logistic_regression_ml_likelihood2(nvars,nsamples,data,model0,varnstates,alphas);
  logL1=logistic_regression_ml_likelihood2(nvars,nsamples,data,model1,varnstates,alphas);
  logratio=logL1-logL0; 
    

  if ((logL0<=(-1*infval))&&(logL1<=(-1*infval)))
    logratio=0;  


  // Count the degree of freedom.
  // If model1 has more input variables than model0, then d=difference of the number of variables.
  // Otherwise d=difference of the number of terms.
  interaction=0;
  d=(model1->X+model1->nvars-1)->npas-(model0->X+model0->nvars-1)->npas;
  if (d==0)
    {
      interaction=1;
      d=(model1->X+model1->nvars-1)->nterms-(model0->X+model0->nvars-1)->nterms;
    }

  // Calculate the chi square p-value.
  if (logratio>0)
    pval1=1-chi2cdf(2*logratio,d);    
  else
    pval1=1;
  
  
  // If not combinatorial interaction, then calculate the permutation p-value.
  if ((interaction==0)&&(pval1<1))
    pval2=approx_perm_pval2(nvars,nsamples,data,model0,model1,varnstates,high_thre,low_thre,nperm);
  else if (interaction==1)
    pval2=0;
  else
    pval2=1;
  
  pval=max(pval1,pval2);
 
  if (logratio<=(-1*infval))
    logratio=0;

  *(retvec+0)=logratio; *(retvec+1)=pval;
 
  free(cnts); free(addinputs); free(alphas);

  return retvec;
}


// Evaluate the log likelihood score of logistic regression model using the maximum likelihood parameters.
// Difference from logistic_regression_ml_likelihood: If the model is independent, then treat it as all lambda coefficient 0.  The log likelihood is - N log(K).  N is the number of samples and K the number of output configurations.
double logistic_regression_ml_likelihood2(int nvars, int nsamples, double *data, struct model *model, int *varnstates, double *alphas)
{
  int i, j, k, independent;
  double *lambdas, logL=0, val;

  // Check whether the model is independent.
  independent=0;
  if ((model->X+model->nvars-1)->npas<=0)
    independent=1;
  else if ((model->X+model->nvars-1)->nterms<=0)
    independent=1;


  // The model is independent: the log likelihood is -Nlog(K).  N: nsamples, K: noutputconfigs.
  if (independent==1)
    {
      k=*(varnstates+nvars-1); val=(double)k;
      logL=-1*nsamples*log(val);
      return logL;
    }
      

  // Estimate parameters.
  lambdas=logistic_regression_parameter_estimation(nvars,nsamples,data,model,varnstates);
 

  // If any component of lambdas<0, then set their values to 0.
  for (i=0; i<=((model->X+model->nvars-1)->nterms-1); i++)
    if (*(lambdas+i)<0)
      *(lambdas+i)=0;


  logL=logistic_regression_ll_evaluation(nvars,nsamples,data,model,varnstates,alphas,lambdas);
 
  free(lambdas); 

  return logL;
}


/* Given input variables of an existing, calculate the log likelihood ratio and pvalue of adding another variable to fit the data.
   Inputs: data, input variables of the existing model, input signs of the existing model, the sign of the additional variable.
   Output: log ratio and pvalue of the joint model compared to the existing model.
   Difference from additional_power: use corrected version of evaluating the log likelihood score of logistic regression.
*/
double * additional_power_corrected(int nsamples, int noldvars, double *olddata, int *oldnstates, int *oldsigns, double *newvardata, int newnstates, int newsign)
{
  int i, j, k, l, m, n, *varnstates;
  double *data, *retvec;
  struct model *model1, *model2;
  
  data=malloc(sizeof(double)*(noldvars+1)*nsamples);  
  for (n=0; n<=(noldvars-2); n++)    
    for (i=0; i<=(nsamples-1); i++)
      *(data+n*nsamples+i)=*(olddata+n*nsamples+i);  
  for (i=0; i<=(nsamples-1); i++)
    *(data+(noldvars-1)*nsamples+i)=*(newvardata+i);
  for (i=0; i<=(nsamples-1); i++)
    *(data+noldvars*nsamples+i)=*(olddata+(noldvars-1)*nsamples+i);
 
  varnstates=malloc(sizeof(int)*(noldvars+1));
  for (n=0; n<=(noldvars-2); n++)
    *(varnstates+n)=*(oldnstates+n);
  *(varnstates+noldvars-1)=newnstates;
  *(varnstates+noldvars)=*(oldnstates+noldvars-1);
    
  // Build the existing model.
  // The last variable of oldvars is the output variable.
  model1=malloc(sizeof(struct model));
  model1->nvars=noldvars+1;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  for (i=0; i<=(model1->nvars-1); i++)
    {
      (model1->X+i)->varind=i;
      (model1->X+i)->npas=0;
      (model1->X+i)->nterms=0;
    }
  (model1->X+model1->nvars-1)->npas=noldvars-1;
  (model1->X+model1->nvars-1)->pas=malloc(sizeof(int)*(model1->X+model1->nvars-1)->npas);
  for (i=0; i<=((model1->X+model1->nvars-1)->npas-1); i++)
    *((model1->X+model1->nvars-1)->pas+i)=i;
  (model1->X+model1->nvars-1)->nterms=noldvars-1;
  (model1->X+model1->nvars-1)->terms=malloc(sizeof(struct term)*(model1->X+model1->nvars-1)->nterms);
  for (n=0; n<=((model1->X+model1->nvars-1)->nterms-1); n++)
    {
      int n1, n2, funcsign, starti;
      ((model1->X+model1->nvars-1)->terms+n)->nvars=1;
      ((model1->X+model1->nvars-1)->terms+n)->varinds=malloc(sizeof(int));
      *(((model1->X+model1->nvars-1)->terms+n)->varinds+0)=n;
      ((model1->X+model1->nvars-1)->terms+n)->nrows=*(varnstates+n);
      n1=*(varnstates+n); n2=2; 
      ((model1->X+model1->nvars-1)->terms+n)->table=malloc(sizeof(int)*n1*n2);
      if (*(varnstates+n)==3)
	starti=-1;
      else
	starti=0;
      k=0; funcsign=*(oldsigns+n);
      for (i=starti; i<=1; i++)
	{
	  k++; 	  
	  *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+0)=i;
	  if (*(varnstates+n)==3)
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=i*funcsign;
	  else
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=funcsign*(2*i-1);
	}
    }

  // Build the joint model.
  // Include the additional variable.
  model2=malloc(sizeof(struct model));
  model2->nvars=model1->nvars;
  model2->X=malloc(sizeof(struct varinfo)*model2->nvars);
  for (n=0; n<=(model2->nvars-1); n++)
    {
      (model2->X+n)->varind=n;
      (model2->X+n)->npas=(model1->X+n)->npas;
      (model2->X+n)->nterms=(model1->X+n)->nterms;
      if ((model2->X+n)->npas>0)
	{
	  int n1, n2, funcsign, starti;

	  (model2->X+n)->npas+=1;
	  (model2->X+n)->pas=malloc(sizeof(int)*(model2->X+n)->npas);
	  for (i=0; i<=((model1->X+n)->npas-1); i++)
	    *((model2->X+n)->pas+i)=*((model1->X+n)->pas+i);
	  *((model2->X+n)->pas+(model2->X+n)->npas-1)=noldvars-1;
	  (model2->X+n)->nterms=(model1->X+n)->nterms+1;
	  (model2->X+n)->terms=malloc(sizeof(struct term)*(model2->X+n)->nterms);
	  for (i=0; i<=((model1->X+n)->nterms-1); i++)
	    {
	      ((model2->X+n)->terms+i)->nvars=((model1->X+n)->terms+i)->nvars;
	      ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int)*((model2->X+n)->terms+i)->nvars);
	      for (j=0; j<=(((model2->X+n)->terms+i)->nvars-1); j++)
		*(((model2->X+n)->terms+i)->varinds+j)=*(((model1->X+n)->terms+i)->varinds+j);
	      ((model2->X+n)->terms+i)->nrows=((model1->X+n)->terms+i)->nrows;
	      n1=((model2->X+n)->terms+i)->nrows; n2=2; 
	      
	      ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2);
	      for (j=0; j<=(n1*n2-1); j++)
		*(((model2->X+n)->terms+i)->table+j)=*(((model1->X+n)->terms+i)->table+j);		
	    }
	  
	  // Add the new variable in another term.
	  i=(model2->X+n)->nterms-1;
	  ((model2->X+n)->terms+i)->nvars=1;
	  ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int));
	  *(((model2->X+n)->terms+i)->varinds+0)=noldvars-1;
	  ((model2->X+n)->terms+i)->nrows=*(varnstates+noldvars-1);
	  n1=*(varnstates+noldvars-1); n2=2;
	  ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2);
	  if (n1==3)
	    starti=-1;
	  else
	    starti=0;
	  k=0; funcsign=newsign;
	  for (j=starti; j<=1; j++)
	    {
	      k++; 	  
	      *(((model2->X+n)->terms+i)->table+(k-1)*n2+0)=j;
	      if (n1==3)
		*(((model2->X+n)->terms+i)->table+(k-1)*n2+1)=j*funcsign;
	      else
		*(((model2->X+n)->terms+i)->table+(k-1)*n2+1)=funcsign*(2*j-1);
	    }	  
	}
    }

  // Calculate the log likelihood and pvalue of the model.
  retvec=ml_llratio_logistic2(noldvars+1,nsamples,data,model2,model1,varnstates);

  // Release memory.
  release_memory_model(model1); 

  release_memory_model(model2); 

  free(data); free(varnstates);


  return retvec;
}


// Evaluate the score of a pairwise interaction model.
// The null model is the additive model with two driver inputs.
double * ml_pairwise_interaction_model_corrected(int nsamples, double *inputvec1, double *inputvec2, double *outputvec, int nstates1, int nstates2, int *direction)
{
  int i, j, k, l, m, n, *varnstates, funcsign;
  double *data, *retvec, *tmpvec, val, *retvec1, *retvec2;
  struct model *model1, *model2;

  *direction=0;
  data=malloc(sizeof(double)*3*nsamples);
  for (i=0; i<=(nsamples-1); i++)
    *(data+i)=*(inputvec1+i);
  for (i=0; i<=(nsamples-1); i++)
    *(data+nsamples+i)=*(inputvec2+i);
  for (i=0; i<=(nsamples-1); i++)
    *(data+nsamples*2+i)=*(outputvec+i);
  
  varnstates=malloc(sizeof(int)*3);
  *(varnstates+0)=nstates1;
  *(varnstates+1)=nstates2;
  *(varnstates+2)=3;

  tmpvec=malloc(sizeof(double)*nsamples);
  for (i=0; i<=(nsamples-1); i++)
    *(tmpvec+i)=*(inputvec1+i)*(*(inputvec2+i));
  
  // Build an additive model with two inputs.
  model1=malloc(sizeof(struct model));
  model1->nvars=3;
  model1->X=malloc(sizeof(struct varinfo)*model1->nvars);
  for (i=0; i<=(model1->nvars-1); i++)
    {
      (model1->X+i)->varind=i;
      (model1->X+i)->npas=0;
      (model1->X+i)->nterms=0;
    }
  (model1->X+model1->nvars-1)->npas=2;
  (model1->X+model1->nvars-1)->pas=malloc(sizeof(int)*(model1->X+model1->nvars-1)->npas);
  for (i=0; i<=((model1->X+model1->nvars-1)->npas-1); i++)
    *((model1->X+model1->nvars-1)->pas+i)=i;
  (model1->X+model1->nvars-1)->nterms=2;
  (model1->X+model1->nvars-1)->terms=malloc(sizeof(struct term)*(model1->X+model1->nvars-1)->nterms);
  for (n=0; n<=((model1->X+model1->nvars-1)->nterms-1); n++)
    {
      int n1, n2, starti;
      double val;
      
      if (n==0)
	val=new_missing_corrcoef(nsamples,inputvec1,outputvec);
      else
	val=new_missing_corrcoef(nsamples,inputvec2,outputvec);
      if (val>=0)
	funcsign=1;
      else
	funcsign=-1;
      ((model1->X+model1->nvars-1)->terms+n)->nvars=1;
      ((model1->X+model1->nvars-1)->terms+n)->varinds=malloc(sizeof(int));
      *(((model1->X+model1->nvars-1)->terms+n)->varinds+0)=n;
      ((model1->X+model1->nvars-1)->terms+n)->nrows=*(varnstates+n);
      n1=*(varnstates+n); n2=2; 
      ((model1->X+model1->nvars-1)->terms+n)->table=malloc(sizeof(int)*n1*n2);
      if (*(varnstates+n)==3)
	starti=-1;
      else
	starti=0;
      k=0; 
      for (i=starti; i<=1; i++)
	{
	  k++; 	  
	  *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+0)=i;
	  if (*(varnstates+n)==3)
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=i*funcsign;
	  else
	    *(((model1->X+model1->nvars-1)->terms+n)->table+(k-1)*n2+1)=funcsign*(2*i-1);
	}
    }
  
  // Build the interaction model.
  // Include the pairwise interaction term.
  // Add the interaction term.
  // There are two possibilities: x1x2 or -x1x2.
  // Evaluate the two models and report the one with a better score.
  model2=malloc(sizeof(struct model));
  model2->nvars=model1->nvars;
  model2->X=malloc(sizeof(struct varinfo)*model2->nvars);

  // Positive sign.
  funcsign=1;
  for (n=0; n<=(model2->nvars-1); n++)
    {
      (model2->X+n)->varind=n;
      (model2->X+n)->npas=(model1->X+n)->npas;
      (model2->X+n)->nterms=(model1->X+n)->nterms;
      if ((model2->X+n)->npas>0)
	{
	  int n1, n2, starti1, starti2, j1, j2, k1, k2;
	  
	  (model2->X+n)->pas=malloc(sizeof(int)*(model2->X+n)->npas);
	  for (i=0; i<=((model1->X+n)->npas-1); i++)
	    *((model2->X+n)->pas+i)=*((model1->X+n)->pas+i);
	  (model2->X+n)->nterms=(model1->X+n)->nterms+1;
	  (model2->X+n)->terms=malloc(sizeof(struct term)*(model2->X+n)->nterms);
	  for (i=0; i<=((model1->X+n)->nterms-1); i++)
	    {
	      ((model2->X+n)->terms+i)->nvars=((model1->X+n)->terms+i)->nvars;
	      ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int)*((model2->X+n)->terms+i)->nvars);
	      for (j=0; j<=(((model2->X+n)->terms+i)->nvars-1); j++)
		*(((model2->X+n)->terms+i)->varinds+j)=*(((model1->X+n)->terms+i)->varinds+j);
	      ((model2->X+n)->terms+i)->nrows=((model1->X+n)->terms+i)->nrows;
	      n1=((model2->X+n)->terms+i)->nrows; n2=2; 	      
	      ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2);
	      for (j=0; j<=(n1*n2-1); j++)
		*(((model2->X+n)->terms+i)->table+j)=*(((model1->X+n)->terms+i)->table+j);
	    }
	  
	  // Add the interaction term.
	  // There are two possibilities: x1x2 or -x1x2.
	  // Evaluate the two models and report the one with a better score.	  
	  i=(model2->X+n)->nterms-1;
	  ((model2->X+n)->terms+i)->nvars=2;
	  ((model2->X+n)->terms+i)->varinds=malloc(sizeof(int)*2);
	  *(((model2->X+n)->terms+i)->varinds+0)=0;
	  *(((model2->X+n)->terms+i)->varinds+1)=1;
	  n1=*(varnstates+0); n2=*(varnstates+1);
	  ((model2->X+n)->terms+i)->nrows=n1*n2;	  
	  ((model2->X+n)->terms+i)->table=malloc(sizeof(int)*n1*n2*3);
	  if (n1==3)
	    starti1=-1;
	  else
	    starti1=0;
	  if (n2==3)
	    starti2=-1;
	  else
	    starti2=0;
	  k=0; 
	  for (j1=starti1; j1<=1; j1++)
	    {
	      for (j2=starti2; j2<=1; j2++)
		{
		  k++; 	  
		  if (n1==3)
		    k1=j1;
		  else
		    k1=2*j1-1;
		  if (n2==3)
		    k2=j2;
		  else
		    k2=2*j2-1;
		 
		  *(((model2->X+n)->terms+i)->table+(k-1)*3+0)=j1;
		  *(((model2->X+n)->terms+i)->table+(k-1)*3+1)=j2;
		  *(((model2->X+n)->terms+i)->table+(k-1)*3+2)=k1*k2*funcsign;		  
		}
	    }	  
	}
    }

  // Calculate the log likelihood and pvalue of the model with positive sign.
  retvec1=ml_llratio_logistic2(3,nsamples,data,model2,model1,varnstates);

  // Change the sign to negative.
  funcsign=-1;
  n=model2->nvars-1; i=(model2->X+n)->nterms-1;
  {
    int n1, n2, starti1, starti2, j1, j2, k1, k2;

    n1=*(varnstates+0); n2=*(varnstates+1);
    if (n1==3)
      starti1=-1;
    else
      starti1=0;
    if (n2==3)
      starti2=-1;
    else
      starti2=0;
    k=0; 
    for (j1=starti1; j1<=1; j1++)
      {
	for (j2=starti2; j2<=1; j2++)
	  {
	    k++;
	    if (n1==3)
	      k1=j1;
	    else
	      k1=2*j1-1;
	    if (n2==3)
	      k2=j2;
	    else
	      k2=2*j2-1;
	    	  
	    *(((model2->X+n)->terms+i)->table+(k-1)*3+0)=j1;
	    *(((model2->X+n)->terms+i)->table+(k-1)*3+1)=j2;
	    *(((model2->X+n)->terms+i)->table+(k-1)*3+2)=k1*k2*funcsign;
	  }
      }
  }

  // Calculate the log likelihood and pvalue of the model with negative sign.
  retvec2=ml_llratio_logistic2(3,nsamples,data,model2,model1,varnstates);

  // Report the one with a better score.
  retvec=malloc(sizeof(double)*2);
  if (*(retvec1+0)>=*(retvec2+0))
    {
      *(retvec+0)=*(retvec1+0);
      *(retvec+1)=*(retvec1+1);
      *direction=1;
    }
  else
    {
      *(retvec+0)=*(retvec2+0);
      *(retvec+1)=*(retvec2+1);
      *direction=-1;
    }

  // Release memory.
  release_memory_model(model1); 

  release_memory_model(model2); 

  free(data); free(varnstates); free(tmpvec);
  free(retvec1); free(retvec2);

  return retvec;
}


// Load clusters.
// Shift the index by 1.
struct integerset * load_cluster(char *filename, int *nclus_int)
{
  int i, j, k, l, m, n, nclus=0, length;
  struct integerset *clus;
  char *s, *item, ch;
  double val;
  FILE *fp;

  clus=malloc(sizeof(struct integerset));
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  if ((fp=fopen(filename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",filename);
      return -1;
    }
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      if (strncmp(s,"cluster",7)==0)
	{
	  nclus++; 
	  clus=realloc(clus,sizeof(struct integerset)*nclus);
	  (clus+nclus-1)->nmembers=0;
	  (clus+nclus-1)->members=malloc(sizeof(int));	  
	}
      else if (strlen(s)>=2)
	{
	  item=getitemval3(s,0,&length,' ');
	  val=atof2(item,length); k=(int)val-1; free(item);	  
	  (clus+nclus-1)->nmembers+=1;
	  (clus+nclus-1)->members=realloc((clus+nclus-1)->members,sizeof(int)*(clus+nclus-1)->nmembers);
	  *((clus+nclus-1)->members+(clus+nclus-1)->nmembers-1)=k;
	}
    }
  fclose(fp);
  free(s);

  *nclus_int=nclus;
  return clus;
}

