// Data structures involved in the logistic regression models.

# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <time.h>


// Data structure of a term in a logistic regression model.
struct term {
  int nvars;  // number of input variables in the term.
  int *varinds; // input variable indices.
  int nrows; // number of rows in the truth table.
  int *table; // truth table of the input-output relation.
};
  

// Data structure of variable information.
struct varinfo {
  int varind;  // variable index.
  int npas;    // number of parents.
  int *pas;    // parent indices.
  int nterms;  // number of terms in the logistic regression.
  struct term *terms;  // terms.
};

// Data structure of a model of logistic regression.
struct model {
  int nvars;  // number of variables.
  struct varinfo *X; // variable information.
};

// Data structure of a gene/mir/segment information.
struct genemirseginfo {
  char *snpname;
  char *name;
  int  chr;
  char *dir;
  long startcoord;
  long endcoord;
};

// Data structure of associations.
struct association {
  int ndrivers;
  char *drivertypes;
  int *driverinds;
  int *directions;
  int nregulators;
  int *regulatorinds;
  int npassengers;
  char *passengertypes;
  int *passengerinds;
  double *logratios;
  double *pvals;
  int *representatives;
};

// Data structure of a matrix.
struct matdata {
  int nprobes;  // number of probes.
  int nsamples; // number of samples.
  struct genemirseginfo *probes; // probe info.
  char *samples; // sample info.
  double *data;  // data.
};

// Data structure of a module.
struct module {
  char *drivertype;  // driver type.
  int ndrivers;  // number of drivers.
  int *driverinds;  // driver indices.
  int direction;    // direction.
  int nregs;        // number of regulators.
  int *reginds;     // regulator indices.
  int npassengers;  // number of passengers.
  int *passengerinds; // passenger indices.
};

// Data structure of integer set.
struct integerset {
  int nmembers;  // number of members.
  int *members;  // members.
};

// Data structure of associations with interaction terms.
struct intassociation {
  int ndrivers;
  char *drivertypes;
  int *driverinds;
  int *directions;
  int nregulators;
  int *regulatorinds;
  int ninteractions;
  char *intdrivertypes;
  int *intdriverinds;
  int *intdirections;
  int *intregulatorinds;
  double *intlogratios;
  double *intpvals;
  int npassengers;
  char *passengertypes;
  int *passengerinds;
  double *logratios;
  double *pvals;
  int *representatives;
};

