/* Utility functions of retrieving and operating data from database files.
 */

# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <time.h>
# include "motif.h"
# include "models.h"
# define MAXLINEWIDTH 1000000
# define NAMELENGTH 30
# define LONGNAMELENGTH 100
// # define LINEWIDTH 30
# define max(i,j) (i>j ? i : j)
# define min(i,j) (i<j ? i : j)
# define argmax(i,j) (j>i ? 2 : 1)

// int fracthre=2;
int fracthre=3;

char * getitemval2(char *s, int ind, int *length);
char * getitemval3(char *s, int ind, int *length, char sepch);
double atof2(char *s, int length);
char * extract_promoterseq(char *refseqinfofilename, char *upstreamfilename, int promoterlength, int nselgenes, char *selgenenames);
int contain_string(char *string1, char *string2);
struct facmotif * extract_facmotif(char *matrixfilename, int *nfacmotifs_pnt);
char * getnextitemval2(char *s, int *curpos, int *length);
struct pwm * extract_pwm(char *matrixfilename, int *nmat_pnt);
char *i2a(long num, int *length);
int pair_cmp(struct pair *p1, struct pair *p2);
int pair_cmp_inv(struct pair *p1, struct pair *p2);
int subseteq(int nset1, int *set1, int nset2, int *set2);
int inlist(int Nv, int *v, int target);
int inlistind(int Nv, int *v, int target);
int *int_union(int nset1, int *set1, int nset2, int *set2, int *nset_pnt);
int isequal(int nset1, int *set1, int nset2, int *set2);
int find_match_motif(char *seq, int seqlength, char *motifseq, int mmnum);
double corrcoef(int nsamples, double *x, double *y);
double covariance(int nsamples, double *x, double *y);
double mean(int nsamples, double *x);
struct component *hierarchical_clustering(int nvars, double *corrs, double highthre, double lowthre, int *nclusters_pnt);
int triplet_cmp(struct triplet *p1, struct triplet *p2);
char * getnextitemval3(char *s, int *length, char sepch, int curpos, int *nextpos);


// Get the designated item in a string.
// Difference from getitemval: items can be separated by tab or space.
char * getitemval2(char *s, int ind, int *length)
{
  int i, tabcnt, curind, cnt;
  char ch, *item, *template;

  template=malloc(sizeof(char)*MAXLINEWIDTH); tabcnt=0; curind=0;

  while ((tabcnt<ind)&&(curind<strlen(s)))
    {
      ch=*(s+curind);

      if ((ch==0x09)||(ch==' '))
	tabcnt++;
      // If the line ends before tabcnt reaching ind, then return null.
      else if ((ch==0x0a)&&(tabcnt<ind))
	tabcnt=ind+1;
      curind++;
    }

  if (tabcnt==ind)
    {      
      cnt=0;
      while (((ch=*(s+curind))!=0x09)&&(ch!=0x0a)&&(ch!=0x0d)&&(ch!=' '))
	{
	  *(template+cnt)=ch;
	  curind++; cnt++;
	}
    
      if (cnt>0)
	{
	  item=malloc(sizeof(char)*(cnt+1));
	  for (i=0; i<=(cnt-1); i++)
	    *(item+i)=*(template+i);

	  *(item+cnt)='\0';

	  *length=cnt;
	}
      else
	*length=cnt;
           
    }
  else
    {
      item=NULL; *length=0;
    }

  free(template);
  return item;
}


// Get the designated item in a string.
// Difference from getitemval2: specify the separator character.
char * getitemval3(char *s, int ind, int *length, char sepch)
{
  int i, tabcnt, curind, cnt;
  char ch, *item, *template;

  template=malloc(sizeof(char)*MAXLINEWIDTH); tabcnt=0; curind=0;

  while ((tabcnt<ind)&&(curind<strlen(s)))
    {
      ch=*(s+curind);

      if (ch==sepch)
	tabcnt++;
      // If the line ends before tabcnt reaching ind, then return null.
      else if ((ch==0x0a)&&(tabcnt<ind))
	tabcnt=ind+1;
      curind++;
    }

  if (tabcnt==ind)
    {
      
      cnt=0;
      while (((ch=*(s+curind))!=sepch)&&(ch!=0x0a)&&(ch!=0x0d))
	{
	  *(template+cnt)=ch;
	  curind++; cnt++;
	}
    
      if (cnt>0)
	{
	  item=malloc(sizeof(char)*cnt);
	  for (i=0; i<=(cnt-1); i++)
	    *(item+i)=*(template+i);
	  *length=cnt;
	}
      else
	*length=cnt;
           
    }
  else
    {
      item=NULL; *length=0;
    }

  free(template);
  
  return item;
}


// Convert a string into a real number.
// Also works for exponential representation.
double atof2(char *s, int length)
{
  double val, expval, power;
  int i, j, cnt, sign, afterpoint, afterexp, expsign;
  char ch;

  val=0; sign=1; cnt=0; afterpoint=0; afterexp=0; 
  expval=0; expsign=1; power=1.0;
  while (cnt<length)
    {
      ch=*(s+cnt);
      if ((cnt==0)&&(ch=='-'))
	sign=-1;
      else if ((ch=='-')&&(afterexp==1))
	expsign=-1;
      else if (ch=='.')
	{
	  afterpoint=1;
	  power=0.1;
	}
      else if ((ch=='e')||(ch=='E'))
	{	
	  afterexp=1;
	  afterpoint=0;
	}
      else if ((ch>='0')&&(ch<='9')&&(afterpoint==0))
	{
	  if (afterexp==0)
	    val=val*10+(ch-'0');
	  else
	    expval=expval*10+(ch-'0');
	}
      else if ((ch>='0')&&(ch<='9')&&(afterpoint==1))
	{
	  if (afterexp==0)
	    val=val+(ch-'0')*power;
	  else
	    expval=expval+(ch-'0')*power;
	  power=power*0.1;
	}
  
      cnt++;
    }
  
  if (sign==-1)
    val=val*(-1);
  if (expsign==-1)
    expval=expval*(-1);

  val=val*pow(10,expval);

  return val;
}


// Extract promoter sequences of a set of genes.
// C code implementation of extract_promoterseq.m.
//void extract_promoterseq(char *refseqinfofilename, char *upstreamfilename, int promoterlength, int nselgenes, char *selgenenames, int *promoterseqs_pnt)
char * extract_promoterseq(char *refseqinfofilename, char *upstreamfilename, int promoterlength, int nselgenes, char *selgenenames)
{
  int i, j, k, l, m, n, length, ntotalgenes, *geneinds, *chrs, *coords, curind, curpos, *visited;
  char *s, ch, *item, *promoterseqs, *genenames, *refseqaccs;
  double val;
  FILE *fp;

  // Load refseq info.
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  if ((fp=fopen(refseqinfofilename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",refseqinfofilename);
      return -1;
    }
  ntotalgenes=0; fgets(s,MAXLINEWIDTH,fp);
  genenames=malloc(sizeof(char)); refseqaccs=malloc(sizeof(char));
  chrs=malloc(sizeof(int)); coords=malloc(sizeof(int));
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      ntotalgenes+=1;
      genenames=realloc(genenames,sizeof(char)*ntotalgenes*NAMELENGTH);
      item=getitemval3(s,1,&length,0x09);
      strncpy(genenames+(ntotalgenes-1)*NAMELENGTH,item,length);
      *(genenames+(ntotalgenes-1)*NAMELENGTH+length)='\0';
      free(item);
      refseqaccs=realloc(refseqaccs,sizeof(char)*ntotalgenes*NAMELENGTH);
      item=getitemval3(s,3,&length,0x09);
      strncpy(refseqaccs+(ntotalgenes-1)*NAMELENGTH,item,length);
      *(refseqaccs+(ntotalgenes-1)*NAMELENGTH+length)='\0';
      free(item);
      item=getitemval3(s,4,&length,0x09);
      val=atof2(item,length);
      chrs=realloc(chrs,sizeof(int)*ntotalgenes);
      *(chrs+ntotalgenes-1)=(int)val;
      free(item);
      item=getitemval3(s,5,&length,0x09);
      val=atof2(item,length);
      coords=realloc(coords,sizeof(int)*ntotalgenes);
      *(coords+ntotalgenes-1)=(int)val;
      free(item);
    }
  fclose(fp);

  
  // Convert gene names into indices.
  geneinds=malloc(sizeof(int)*nselgenes);
  for (i=0; i<=(nselgenes-1); i++)
    {
      j=0; k=-1;
      while ((j<ntotalgenes)&&(k<0))
	{
	  if (strncmp(selgenenames+i*NAMELENGTH,genenames+j*NAMELENGTH,max(strlen(selgenenames+i*NAMELENGTH),strlen(genenames+j*NAMELENGTH)))==0)
	    k=j;
	  j++;
	}
      *(geneinds+i)=k;
    }
  
  // Search the upstream file.
  promoterseqs=malloc(sizeof(char)*nselgenes*promoterlength);
  memset(promoterseqs,'-',nselgenes*promoterlength);  
  
  if ((fp=fopen(upstreamfilename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",upstreamfilename);
      return -1;
    }
  
  curind=-1; curpos=0; visited=malloc(sizeof(int)*nselgenes);
  for (i=0; i<=(nselgenes-1); i++)
    *(visited+i)=0;
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      // Get the refseq accession number.  Match it to a selected gene.
      if (*s=='>')
	{
	  int startpos=-1, endpos=-1;	 
	  
	  startpos=1; i=4;
	  while ((i<strlen(s))&&(endpos<0))
	    {
	      if ((ch=*(s+i))=='_')
		endpos=i-1;
	      i++;
	    }

	  
	  curind=-1;
	  if ((startpos>=0)&&(endpos>=startpos))
	    {	      
	      i=0; k=-1;
	      while ((i<nselgenes)&&(k<0))
		{
		  j=*(geneinds+i);		  
		  if ((j>=0)&&(j<ntotalgenes)&&(*(visited+i)==0)&&(strncmp(s+startpos,refseqaccs+j*NAMELENGTH,max(endpos-startpos+1,strlen(refseqaccs+j*NAMELENGTH)))==0))
		    k=i;
		  i++;
		}
	      if (k>=0)
		{
		  curind=k; curpos=0;
		}
	    }
	  
	}

      // Write the sequence.
      else if ((*s!='>')&&(curind>=0))
	{		  
	  strncpy(promoterseqs+curind*promoterlength+curpos,s,strlen(s)-1);
	  curpos+=(strlen(s)-1);	  	 
	}
    }
  fclose(fp); 
  
  return promoterseqs;
}


// Extract the position weight matrices from transfac.
struct pwm * extract_pwm(char *matrixfilename, int *nmat_pnt)
{
  int i, j, k, l, m, n, length, nmat, matcnt;
  char *s, ch, *item;
  double val;
  struct pwm *pwmat;
  FILE *fp;

  // Open the pwm file.
  if ((fp=fopen(matrixfilename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",matrixfilename);
      return -1;
    }
  s=malloc(sizeof(char)*MAXLINEWIDTH);
  pwmat=malloc(sizeof(struct pwm)); matcnt=-1; nmat=0;
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      int startpos, endpos;

      // The beginning of a matrix record.
      if (strncmp(s,"AC  ",4)==0)
	{
	  nmat++; pwmat=realloc(pwmat,sizeof(struct pwm)*nmat);	  
	  startpos=4; endpos=strlen(s)-2;
	  (pwmat+nmat-1)->acc=malloc(sizeof(char)*NAMELENGTH);
	  strncpy((pwmat+nmat-1)->acc,s+startpos,endpos-startpos+1);
	  *((pwmat+nmat-1)->acc+endpos-startpos+1)='\0';
	  (pwmat+nmat-1)->nbfs=0; (pwmat+nmat-1)->npos=0;
	  (pwmat+nmat-1)->bfs=malloc(sizeof(char));
	  (pwmat+nmat-1)->mat=malloc(sizeof(double));	  
	}

      // Matrix name.
      else if (strncmp(s,"NA  ",4)==0)
	{	  
	  (pwmat+nmat-1)->name=malloc(sizeof(char)*NAMELENGTH);
	  startpos=4; endpos=strlen(s)-2;
	  strncpy((pwmat+nmat-1)->name,s+startpos,endpos-startpos+1);
	  *((pwmat+nmat-1)->name+endpos-startpos+1)='\0';	   
	}

      // Binding factors.
      else if (strncmp(s,"BF  ",4)==0)
	{
	  startpos=4; endpos=9;
	  (pwmat+nmat-1)->nbfs+=1;
	  (pwmat+nmat-1)->bfs=realloc((pwmat+nmat-1)->bfs,sizeof(char)*(pwmat+nmat-1)->nbfs*NAMELENGTH);
	  strncpy((pwmat+nmat-1)->bfs+((pwmat+nmat-1)->nbfs-1)*NAMELENGTH,s+startpos,endpos-startpos+1);
	  *((pwmat+nmat-1)->bfs+((pwmat+nmat-1)->nbfs-1)*NAMELENGTH+endpos-startpos+1)='\0';	  
	}

      // Matrix start.
      else if (strncmp(s,"P0 ",3)==0)
	matcnt=0;
      
      // Matrix end.
      else if ((matcnt>=0)&&(strncmp(s,"XX",2)==0))
	matcnt=-1;

      // Matrix entry.
      else if (matcnt>=0)
	{
	  // Get 4 numbers.
	  int cnts[4], cnt;
	  i=3; cnt=0; startpos=-1; endpos=-1;
	  while ((i<strlen(s))&&(cnt<=3))
	    {
	      ch=*(s+i);
	      if ((ch>='0')&&(ch<='9')&&(startpos<0))
		startpos=i;
	      else if ((ch==' ')&&(startpos>=0))
		{
		  endpos=i-1; 
		  val=atof2(s+startpos,endpos-startpos+1);
		  cnts[cnt]=(int)val; cnt++;
		  startpos=-1;
		}
	      i++;
	    }	 	  

	  // Write the numbers to the matrix.
	  (pwmat+nmat-1)->npos+=1;
	  (pwmat+nmat-1)->mat=realloc((pwmat+nmat-1)->mat,sizeof(double)*(pwmat+nmat-1)->npos*4);
	  val=0;
	  for (i=0; i<=3; i++)
	    val+=(double)cnts[i];
	  for (i=0; i<=3; i++)
	    *((pwmat+nmat-1)->mat+((pwmat+nmat-1)->npos-1)*4+i)=(double)cnts[i]/val;	  	  
	}
    }
  fclose(fp);

  // Return the matrices.  
  free(s);

  *nmat_pnt=nmat; return pwmat;
  
}


// Convert an integer into a string.
char *i2a(long num, int *length)
{
  int i, j, k;
  double ee;
  char *item;

  *length=0; item=malloc(sizeof(char));
  if (num<0)
    {
      *(item+*length-1)='-'; *length+=1;
      item=realloc(item,sizeof(char)*(*length));
      num*=(-1);
    }

  if (num!=0)
    {
      ee=log10((double)num); j=(int)ee+1;
      *length+=j; item=realloc(item,sizeof(char)*(*length));
    }
  else
    {
      *length=1; j=1;
    }
  for (i=0; i<=(j-1); i++)
    {
      k=num%10; *(item+*length-1-i)='0'+k; num=(num-k)/10;
    }

  return item;
}



// Compare two pairs.  Place a large one in the front.
int pair_cmp(struct pair *p1, struct pair *p2)
{   
  if (p1->j>p2->j)
    return -1;
  else if (p1->j<p2->j)
    return 1;
  else
    return 0;  
}


// Compare two pairs.  Place a small one in the front.
int pair_cmp_inv(struct pair *p1, struct pair *p2)
{
  if (p1->j>p2->j)
    return 1;
  else if (p1->j<p2->j)
    return -1;
  else
    return 0;
}

// Compare two triplets.  Place a large one in the front.
int triplet_cmp(struct triplet *p1, struct triplet *p2)
{   
  if (p1->s>p2->s)
    return -1;
  else if (p1->s<p2->s)
    return 1;
  else
    return 0;  
}

// Check whether set1 is a subset or equal to set2.
int subseteq(int nset1, int *set1, int nset2, int *set2)
{
  int i, flag;
  
  i=0; flag=1;
  while ((i<nset1)&&(flag==1))
    {
      if (inlist(nset2,set2,*(set1+i))==0)
	flag=0;
      i++;
    }
  return flag;
}


// Check if two sets are equal.
int isequal(int nset1, int *set1, int nset2, int *set2)
{
  if (nset1!=nset2)
    return 0;
  
  if ((subseteq(nset1,set1,nset2,set2)==1)&&(subseteq(nset2,set2,nset1,set1)==1))
    return 1;
  else
    return 0;
}


// Check if a number is on a list.
int inlist(int Nv, int *v, int target)
{
  int i=0, indicator=0;

  while ((i<Nv)&&(indicator==0))
    {
      if (*(v+i)==target)
	indicator=1;
      i++;
    }

  return indicator;
}

// Obtain the index of a number on a list.
int inlistind(int Nv, int *v, int target)
{
  int i=0, ind=-1;

  while ((i<Nv)&&(ind==-1))
    {
      if (*(v+i)==target)
	ind=i;
      i++;
    }

  return ind;
}

// Union of a set.
int *int_union(int nset1, int *set1, int nset2, int *set2, int *nset_pnt)
{
  int i, j, nset, *set;
  
  nset=nset1; set=malloc(sizeof(int)*(nset1+nset2));
  for (i=0; i<=(nset1-1); i++)    
    *(set+i)=*(set1+i);
  
  for (i=0; i<=(nset2-1); i++)
    {
      j=*(set2+i);
      if (inlist(nset,set,j)==0)
	{
	  nset++; *(set+nset-1)=j;
	}
    }
  
  *nset_pnt=nset;
  return set;
}


// Find whether a motif appears in a sequence.
// Allow certain number of mismatches.
int find_match_motif(char *seq, int seqlength, char *motifseq, int mmnum)
{
  int i, j, length, mm, flag;
  
  flag=0; i=0; length=strlen(motifseq);  
  while ((i<(seqlength-length))&&(flag==0))
    {
      mm=0;
      for (j=0; j<=(length-1); j++)
	if (*(seq+i+j)!=*(motifseq+j))
	  mm++;
      if (mm<=mmnum)	
	flag=1;	     

      i++;
    }

  return flag;
}


// Extract the motifs of a factor from the pwm file.
struct facmotif * extract_facmotif(char *matrixfilename, int *nfacmotifs_pnt)
{
  int i, j, k, l, m, n, length, nfacmotifs, nmotifs, matcnt;
  char *s, ch, *item;
  double val;
  struct facmotif *facmotifs;
  struct motif *motifs;
  FILE *fp;

  // Open the pwm file.  Load motifs.
  if ((fp=fopen(matrixfilename,"r"))==NULL)
    {
      printf("Cannot open %s.\n",matrixfilename);
      return -1;
    }
  s=malloc(sizeof(char)*MAXLINEWIDTH); nmotifs=0; motifs=malloc(sizeof(struct motif)); matcnt=-1;
  while ((fgets(s,MAXLINEWIDTH,fp))!=NULL)
    {
      int startpos, endpos;

      // The beginning of a matrix record.
      if (strncmp(s,"AC  ",4)==0)
	{
	  nmotifs++; motifs=realloc(motifs,sizeof(struct motif)*nmotifs);
	  startpos=4; endpos=strlen(s)-2;
	  (motifs+nmotifs-1)->acc=malloc(sizeof(char)*NAMELENGTH);
	  strncpy((motifs+nmotifs-1)->acc,s+startpos,endpos-startpos+1);
	  *((motifs+nmotifs-1)->acc+endpos-startpos+1)='\0';
	  (motifs+nmotifs-1)->nbfs=0; 
	  (motifs+nmotifs-1)->facaccs=malloc(sizeof(char)*NAMELENGTH);
	  (motifs+nmotifs-1)->facnames=malloc(sizeof(char)*NAMELENGTH);
	  (motifs+nmotifs-1)->consensus=malloc(sizeof(char)*NAMELENGTH);	 
	}
      
      // Factor accs and names.
      else if (strncmp(s,"BF  ",4)==0)
	{
	  int ff=0;

	  // Check if it is a human factor.
	  i=0; 
	  while ((i<(strlen(s)-13))&&(ff==0))
	    {
	      if (strncmp(s+i,"Homo sapiens",12)==0)
		ff=1;
	      i++;
	    }

	  if (ff==1)
	    {
	      i=0; startpos=-1;
	      while ((i<strlen(s))&&(startpos<0))
		{
		  if ((ch=*(s+i))==';')
		    {
		      startpos=i-6; endpos=i-1;
		    }
		  i++;
		}
	      if (startpos>=0)
		{
		  (motifs+nmotifs-1)->nbfs+=1;		  
		  (motifs+nmotifs-1)->facaccs=realloc((motifs+nmotifs-1)->facaccs,sizeof(char)*(motifs+nmotifs-1)->nbfs*NAMELENGTH);
		  strncpy((motifs+nmotifs-1)->facaccs+((motifs+nmotifs-1)->nbfs-1)*NAMELENGTH,s+startpos,endpos-startpos+1);
		  *((motifs+nmotifs-1)->facaccs+((motifs+nmotifs-1)->nbfs-1)*NAMELENGTH+endpos-startpos+1)='\0';
		  
		  i=endpos+2; startpos=-1;
		  while ((i<strlen(s))&&(startpos<0))
		    {
		      if ((ch=*(s+i))!=' ')
			startpos=i;
		      i++;
		    }
		  i=startpos; endpos=-1;
		  while ((i<strlen(s))&&(endpos<0))
		    {
		      if ((ch=*(s+i))==';')
			endpos=i-1;
		      i++;
		    }
		  (motifs+nmotifs-1)->facnames=realloc((motifs+nmotifs-1)->facnames,sizeof(char)*(motifs+nmotifs-1)->nbfs*NAMELENGTH);
		  strncpy((motifs+nmotifs-1)->facnames+((motifs+nmotifs-1)->nbfs-1)*NAMELENGTH,s+startpos,endpos-startpos+1);
		  *((motifs+nmotifs-1)->facnames+((motifs+nmotifs-1)->nbfs-1)*NAMELENGTH+endpos-startpos+1)='\0';
		  		  
		}
	    }	  
	}

      // Consensus sequences.

      // Matrix start.
      else if ((strncmp(s,"P0 ",3)==0)&&((motifs+nmotifs-1)->nbfs>0))	
	matcnt=0;
	  
      // Matrix end.
      else if ((matcnt>=0)&&(strncmp(s,"XX",2)==0)&&((motifs+nmotifs-1)->nbfs>0))
	{
	  *((motifs+nmotifs-1)->consensus+matcnt)='\0';
	  matcnt=-1;	 
	}

      // Matrix entry.
      else if ((matcnt>=0)&&((motifs+nmotifs-1)->nbfs>0))
	{
	  // Get the last column: consensus sequence.
	  ch=*(s+strlen(s)-2); *((motifs+nmotifs-1)->consensus+matcnt)=ch;	  
	  matcnt++;
	}
    }  
 
  // Group the motifs of the same TF together.
  nfacmotifs=0; facmotifs=malloc(sizeof(struct facmotif));
  for (n=0; n<nmotifs; n++)
    {
      for (i=0; i<(motifs+n)->nbfs; i++)
	{
	  j=0; k=-1;
	  while ((j<nfacmotifs)&&(k<0))
	    {
	      if (strncmp((motifs+n)->facaccs+i*NAMELENGTH,(facmotifs+j)->facacc,max(strlen((motifs+n)->facaccs+i*NAMELENGTH),strlen((facmotifs+j)->facacc)))==0)
		k=j;
	      j++;
	    }
	  if (k<0)
	    {
	      nfacmotifs+=1; facmotifs=realloc(facmotifs,sizeof(struct facmotif)*nfacmotifs);
	      (facmotifs+nfacmotifs-1)->facacc=malloc(sizeof(char)*NAMELENGTH);
	      strncpy((facmotifs+nfacmotifs-1)->facacc,(motifs+n)->facaccs+i*NAMELENGTH,NAMELENGTH-1);
	      (facmotifs+nfacmotifs-1)->facname=malloc(sizeof(char)*NAMELENGTH);
	      strncpy((facmotifs+nfacmotifs-1)->facname,(motifs+n)->facnames+i*NAMELENGTH,NAMELENGTH-1);
	      (facmotifs+nfacmotifs-1)->nmotifs=1;
	      (facmotifs+nfacmotifs-1)->motifaccs=malloc(sizeof(char)*NAMELENGTH);
	      strncpy((facmotifs+nfacmotifs-1)->motifaccs,(motifs+n)->acc,NAMELENGTH-1);
	      (facmotifs+nfacmotifs-1)->motifs=malloc(sizeof(char)*NAMELENGTH);
	      strncpy((facmotifs+nfacmotifs-1)->motifs,(motifs+n)->consensus,NAMELENGTH-1);	      

	    }
	  else
	    {
	      (facmotifs+k)->nmotifs+=1;
	      (facmotifs+k)->motifaccs=realloc((facmotifs+k)->motifaccs,sizeof(char)*(facmotifs+k)->nmotifs*NAMELENGTH);
	      strncpy((facmotifs+k)->motifaccs+((facmotifs+k)->nmotifs-1)*NAMELENGTH,(motifs+n)->acc,NAMELENGTH-1);
	      (facmotifs+k)->motifs=realloc((facmotifs+k)->motifs,sizeof(char)*(facmotifs+k)->nmotifs*NAMELENGTH);
	      strncpy((facmotifs+k)->motifs+((facmotifs+k)->nmotifs-1)*NAMELENGTH,(motifs+n)->consensus,NAMELENGTH-1);
	    }
	}
    }

  // Release the memory of motifs.
  for (n=0; n<nmotifs; n++)
    {
      free((motifs+n)->acc);
      free((motifs+n)->facaccs);
      free((motifs+n)->facnames);
      free((motifs+n)->consensus);
    }
  free(motifs); free(s);

  *nfacmotifs_pnt=nfacmotifs;
  
  return facmotifs;
}


// Get the designated item in a string.
// Difference from getitemval: items can be separated by tab or space.
// Difference from getitemval2: specify and modify the current position.
char * getnextitemval2(char *s, int *curpos, int *length)
{
  int i, curind, cnt;
  char ch, *item, *template;  

  template=malloc(sizeof(char)*MAXLINEWIDTH); 

  cnt=0; curind=*curpos;
  while (((ch=*(s+curind))!=0x09)&&(ch!=0x0a)&&(ch!=0x0d)&&(ch!=' '))
    {
      *(template+cnt)=ch;
      curind++; cnt++;
    }
    
  if (cnt>0)
    {
      item=malloc(sizeof(char)*(cnt+1));
      for (i=0; i<=(cnt-1); i++)
	*(item+i)=*(template+i);

      *(item+cnt)='\0';
      
      *length=cnt; 

      if ((ch==0x09)||(ch==' '))
	*curpos=curind+1;
      else
	*curpos=curind;
    }
  else    
    *length=cnt;
  
  free(template);

  return item;
}


// Compute correlation coefficient of two vectors.
double corrcoef(int nsamples, double *x, double *y)
{
  double val1, val2, val3, val;
  val1=covariance(nsamples,x,y); 
  val2=covariance(nsamples,x,x); val3=covariance(nsamples,y,y);
  if ((val2*val3)>0)
    val=val1/sqrt(val2*val3);
  else
    val=0;
  return val;
}

// Compute the covariance function of two vectors.
double covariance(int nsamples, double *x, double *y)
{
  int i;
  double mx, my, val, val2;
  
  mx=mean(nsamples,x); my=mean(nsamples,y); val=0;
  for (i=0; i<=(nsamples-1); i++)
    {
      val2=(*(x+i)-mx)*(*(y+i)-my);
      val+=val2;
    }
  val/=nsamples;
  return val;
}

// Compute the mean of a vector.
double mean(int nsamples, double *x)
{
  int i;
  double val=0;
  for (i=0; i<=(nsamples-1); i++)
    val+=*(x+i);
  val/=nsamples;
  return val;
}


// Apply hierarchical clustering on the correlation matrix.
struct component *hierarchical_clustering(int nvars, double *corrs, double highthre, double lowthre, int *nclusters_pnt)
{
  int i, j, k, l, m, n, length, nclus, nclusters, iteration;
  struct pair *ns;
  struct component *clusters, *clus;  
  
  // Initialize clusters.
  nclus=nvars; clus=malloc(sizeof(struct component)*nclus);
  for (i=0; i<=(nclus-1); i++)
    {
      (clus+i)->nmembers=1;
      (clus+i)->members=malloc(sizeof(int));
      *((clus+i)->members+0)=i;
    }

  // Hierarchically merge clusters.
  iteration=1;
  while (iteration==1)
    {
      int nmg, *mg, *inds;
      double *gap;
      
      // Find all mergeable cluster pairs.
      nmg=0; mg=malloc(sizeof(int)); gap=malloc(sizeof(double));
      inds=malloc(sizeof(int)*nvars);
      for (i=0; i<=(nclus-2); i++)
	{
	  for (j=(i+1); j<=(nclus-1); j++)
	    {
	      int n1, n2;
	      double *subcorrs, minv, maxv;

	      for (k=0; k<=((clus+i)->nmembers-1); k++)
		*(inds+k)=*((clus+i)->members+k);
	      l=(clus+i)->nmembers;
	      for (k=0; k<=((clus+j)->nmembers-1); k++)
		*(inds+l+k)=*((clus+j)->members+k);
	      k=(clus+i)->nmembers+(clus+j)->nmembers;
	     
	      // Calculate the subcorrelation matrix.
	      subcorrs=malloc(sizeof(double)*k*k);
	      for (l=0; l<=(k-1); l++)
		{
		  for (m=0; m<=(k-1); m++)
		    {
		      *(subcorrs+l*k+m)=*(corrs+*(inds+l)*nvars+*(inds+m));		      
		    }
		}	     

	      // Count the number of entries passing or failing the thresholds.
	      n1=0; n2=0; minv=1.0; maxv=-1.0;
	      for (l=0; l<=((clus+i)->nmembers-1); l++)
		{
		  for (m=0; m<=((clus+j)->nmembers-1); m++)
		    {
		      if (*(subcorrs+l*k+m+(clus+i)->nmembers)>=highthre)
			n1++;
		      else if (*(subcorrs+l*k+m+(clus+i)->nmembers)<=lowthre)
			n2++;
		      
		      if (*(subcorrs+l*k+m+(clus+i)->nmembers)>maxv)
			maxv=*(subcorrs+l*k+m+(clus+i)->nmembers);
		      if (*(subcorrs+l*k+m+(clus+i)->nmembers)<minv)
			minv=*(subcorrs+l*k+m+(clus+i)->nmembers);
		    }
		}
	      
	      if (((n1*fracthre)>=((clus+i)->nmembers*(clus+j)->nmembers))&&((n2*10)<=((clus+i)->nmembers*(clus+j)->nmembers)))
		{
		  nmg++; mg=realloc(mg,sizeof(int)*nmg*3);
		  *(mg+(nmg-1)*3+0)=i; *(mg+(nmg-1)*3+1)=j; 
		  *(mg+(nmg-1)*3+2)=n1-n2;
		  
		  gap=realloc(gap,sizeof(double)*nmg);
		  *(gap+nmg-1)=maxv+minv;
		}

	      free(subcorrs);
	    }
	}
      free(inds);
     
      // Merge clusters or stop.
      if (nmg==0)
	iteration=0;

      else
	{
	  int maxind=-1, maxval=-100;
	  double fmaxval=-100;

	  for (i=0; i<=(nmg-1); i++)
	    {
	      if (*(gap+i)>fmaxval)
		{
		  maxind=i; fmaxval=*(gap+i);
		}

	    }
	  i=*(mg+maxind*3+0); j=*(mg+maxind*3+1);

	  // Debug
	  printf("nclus=%d, ",nclus);
	  printf("( ");
	  for (k=0; k<=((clus+i)->nmembers-1); k++)
	    printf("%d ",*((clus+i)->members+k)+1);
	  printf(")( ");
	  for (k=0; k<=((clus+j)->nmembers-1); k++)
	    printf("%d ",*((clus+j)->members+k)+1);
	  printf(")\n");

	  
	  // Merge clusters j to i.
	  (clus+i)->members=realloc((clus+i)->members,sizeof(int)*((clus+i)->nmembers+(clus+j)->nmembers));
	  for (k=0; k<=((clus+j)->nmembers-1); k++)
	    *((clus+i)->members+(clus+i)->nmembers+k)=*((clus+j)->members+k);
	  (clus+i)->nmembers+=(clus+j)->nmembers;

	  // Shift all the clusters after j.
	  free((clus+j)->members);
	  for (k=(j+1); k<=(nclus-1); k++)
	    {
	      (clus+k-1)->nmembers=(clus+k)->nmembers;
	      (clus+k-1)->members=(clus+k)->members;
	    }
	  nclus--;
	}

      free(mg); free(gap);
    }

  // Sort clusters by size.
  ns=malloc(sizeof(struct pair)*nclus);
  for (i=0; i<=(nclus-1); i++)
    {
      (ns+i)->i=i; (ns+i)->j=(clus+i)->nmembers;
    }

  qsort(ns,nclus,sizeof(struct pair),(* pair_cmp));
    

  // Report the sorted clusters.
  nclusters=nclus;
  clusters=malloc(sizeof(struct component)*nclusters);
  for (n=0; n<=(nclusters-1); n++)
    {
      i=(ns+n)->i;
      (clusters+n)->nmembers=(clus+i)->nmembers;
      (clusters+n)->members=(clus+i)->members;      
    }
  
  // Debug
  printf("nclus=%d\n",nclus);
  for (n=0; n<=(nclus-1); n++)
    {
      if ((clusters+n)->nmembers>1)
	{
	  printf("clu %d has %d members: ",n,(clusters+n)->nmembers);
	  for (i=0; i<=((clusters+n)->nmembers-1); i++)
	    printf("%d ",*((clusters+n)->members+i)+1);
	  printf("\n");
	}
    }
  

  free(ns); *nclusters_pnt=nclusters;
  

  return clusters;
}


// Get an item starting at position startpos.  Return the startpos of the next item.
char * getnextitemval3(char *s, int *length, char sepch, int curpos, int *nextpos)
{
  int i;
  char ch, *item;

  *length=0;
  i=curpos; *nextpos=-1;
  while ((i<strlen(s))&&(*nextpos<0))
    {
      if (((ch=*(s+i))==sepch)||(ch==0x0d))
	*nextpos=i+1;
      else
	*length+=1;
      i++;
    }
  if (*length>0)
    {
      item=malloc(sizeof(char)*(*length+1));
      strncpy(item,s+curpos,*length);
      *(item+*length)='\0';
    }
  return item;
}

// Check whether string1 contains string2.
int contain_string(char *string1, char *string2)
{
  int i, flag, retval;

  if (strlen(string1)<strlen(string2))
    {
      retval=0; return retval;
    }

  i=0; flag=0; retval=0;
  while ((i<(strlen(string1)-1))&&(flag==0))
    {
      if (strncmp(string1+i,string2,strlen(string2))==0)
	flag=1;
      i++;
    }
  if (flag==1)
    retval=1;
  
  return retval;
}
