/* Utility functions of graphs.
 */

# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <time.h>
# include "motif.h"
# define MAXLINEWIDTH 1000000
# define NAMELENGTH 30
# define LONGNAMELENGTH 100
// # define LINEWIDTH 30
# define max(i,j) (i>j ? i : j)
# define min(i,j) (i<j ? i : j)
# define argmax(i,j) (j>i ? 2 : 1)

extern int pair_cmp(struct pair *p1, struct pair *p2);
extern int pair_cmp_inv(struct pair *p1, struct pair *p2);
extern int subseteq(int nset1, int *set1, int nset2, int *set2);
extern int inlist(int Nv, int *v, int target);
extern int *int_union(int nset1, int *set1, int nset2, int *set2, int *nset_pnt);
extern int isequal(int nset1, int *set1, int nset2, int *set2);
extern char * getitemval2(char *s, int ind, int *length);
extern char *i2a(long num, int *length);
extern double atof2(char *s, int length);
extern char * getitemval3(char *s, int ind, int *length, char sepch);
extern double corrcoef(int nsamples, double *x, double *y);
extern char * getnextitemval2(char *s, int *curpos, int *length);
extern double mean(int nsamples, double *x);
extern int triplet_cmp(struct triplet *p1, struct triplet *p2);

int * find_undir_conn_comps(int nnodes, int *conn);
void recursive_labeling(int nnodes, int *conn, int *labels, int targetnode, int targetlabel);


// Find connected components of an undirected graph.
// Return the component labels of nodes.
int * find_undir_conn_comps(int nnodes, int *conn)
{
  int i, j, k, l, m, n, *labels, ncomps, *newlabels, flag;
  struct pair *pairs;
  
  
  labels=malloc(sizeof(int)*nnodes);
  for (i=0; i<=(nnodes-1); i++)
    *(labels+i)=-1;
  
  // Label all the singletons.
  n=0;
  for (i=0; i<=(nnodes-1); i++)
    {
      k=0;
      for (j=0; j<=(nnodes-1); j++)	
	if ((j!=i)&&(*(conn+i*nnodes+j)>0))
	  k++;      
      if (k==0)
	{
	  *(labels+i)=n; n++;
	}
    }

  flag=1;
  while (flag==1)
    {
      // Check if all nodes are labeled.
      k=0;
      for (i=0; i<=(nnodes-1); i++)	
	if (*(labels+i)<0)
	  k++;
      
      // If yes then return.
      if (k==0)
	flag=0;

      // Otherwise pick up an unlabeled node and recursively label its neighbors.
      else
	{
	  i=0; j=-1;
	  while ((i<=(nnodes-1))&&(j<0))
	    {
	      if (*(labels+i)<0)
		j=i;
	      i++;
	    }	  	  
	  recursive_labeling(nnodes,conn,labels,j,n);
	  n++;
	}
    }


  // Sort the labels by the component sizes.
  ncomps=0; 
  for (i=0; i<=(nnodes-1); i++)
    {
      j=*(labels+i);
      ncomps=max(ncomps,j+1);
    }
  pairs=malloc(sizeof(struct pair)*ncomps);   
  for (i=0; i<=(ncomps-1); i++)
    {
      (pairs+i)->i=i; (pairs+i)->j=0; 
    }
  for (i=0; i<=(nnodes-1); i++)
    {
      j=*(labels+i);
      (pairs+j)->j+=1;
    }
  qsort(pairs,ncomps,sizeof(struct pair),(* pair_cmp));
  
  // Return the sorted labels.
  newlabels=malloc(sizeof(int)*nnodes);
  for (i=0; i<=(nnodes-1); i++)
    {
      j=*(labels+i); k=0; l=-1;
      while ((k<=(ncomps-1))&&(l<0))
	{
	  if ((pairs+k)->i==j)
	    l=k;
	  k++;
	}
      *(newlabels+i)=l;
    }

  free(labels); free(pairs);
  return newlabels;
}


// Recursively label the nodes in an undirected graph.
void recursive_labeling(int nnodes, int *conn, int *labels, int targetnode, int targetlabel)
{
  int i, j, k, l, n;

  // Label the target node.
  if ((targetnode>=0)&&(targetnode<nnodes))
    {
      *(labels+targetnode)=targetlabel;
    }

  // Recursively visit the neighbors and label the unlabeled ones.
  for (i=0; i<=(nnodes-1); i++)    
    {
      if ((i!=targetnode)&&(*(conn+targetnode*nnodes+i)>0)&&(*(labels+i)<0))	
	{
	  recursive_labeling(nnodes,conn,labels,i,targetlabel);
	}
    }
	  
  return;
}




