% Recursively partition the probe data sorted by chromosomal coordinates.
% Compute the likelihood of a binary partition.  Assume the variables at one side are repeated samples of the same hidden variable, which are represented by the consensus.
% Stop when there is only one variable or when partition yields worse likelihood compared to the model that both sides are generated by the same variable.
% Difference from recurse_partition.m and recurse_partition2.m: the probe data are fractional counts of tristates.

function labels = recurse_partition3(fraccnts, labels, startind, endind, numthre, valthre)


% Stop if there is only one variable.
% Stop if the number of probes in the interval is smaller than a threshold.
if ((endind-startind+1)>=numthre)

% Calculate the log likelihood of no partition.
l0=naive_Bayes_loglikelihood2(fraccnts(startind:endind,:));

% For each partition, calculate the log likelihood.
ls=-inf*ones(1,endind);
for i=startind:(endind-1)
l1=naive_Bayes_loglikelihood2(fraccnts(startind:i,:));
l2=naive_Bayes_loglikelihood2(fraccnts((i+1):endind,:));
ls(i)=l1+l2;
end


% Apply the partition if the best likelihood is better than no partition.
if (max(ls)>(l0+valthre))
k=find(ls>=max(ls)); k=k(1);

labels((k+1):endind)=max(labels)+1;

labels=recurse_partition3(fraccnts,labels,startind,k,numthre,valthre);
labels=recurse_partition3(fraccnts,labels,k+1,endind,numthre,valthre);

end
end


